<?php
/*
 * the home page
 */
require_once("include.php");
InputPage::$method = "post";

class ButtonUploadImage extends InputPageAction {
    
    public function ButtonUploadImage($name, $label) {
        parent::__construct($name, $label);
        $this->autorender = false;
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $file = $page->getField("upload");
        if(!$file->valid()) {
            $page->render();
            return;
        }
        if(!move_uploaded_file($file->status->path, Config::imagesDir."/".$file->status->name)) {
            echo "Error: Cannot move image ".$file->status->name." to images directory";
            return;
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&imageuploaded=".urlencode($file->status->name));
    }    
}

class ButtonLdapConfig extends MainInputPageOkCancelButton {
    public function ButtonLdapConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $tls = $page->getField("tls");
        $state = $page->getField("state");
        $pbx = $page->getField("pbx");
        $pbx->value = trim($pbx->value);
        $host = $page->getField("ldaphost");
        $port = $page->getField("ldapport");
        $password = $page->getField("ldappassword");
        $user = $page->getField("ldapuser");
        $user->value = trim($user->value);
        $base = $page->getField("base");
        $base->value = trim($base->value);
        $filter = $page->getField("filter");
        $filter->value = trim($filter->value);
        $nameAttrib = $page->getField("nameattrib");
        $nameAttrib->value = str_replace(" ", "", $nameAttrib->value);
        $numberAttrib = $page->getField("numberattrib");
        $numberAttrib->value = str_replace(" ", "", $numberAttrib->value);
        $cc = $page->getField("countrycode");
        $ac = $page->getField("areacode");
        $np = $page->getField("nationalprefix");
        $ip = $page->getField("internationalprefix");
        $el = $page->getField("extline");
        $sn = $page->getField("subsnumbers");
        $error = "";

        if(!$host->valid() || !$pbx->valid() || /*!$password->valid() || !$user->valid() ||*/ !$base->valid() || !$port->valid() || $nameAttrib->value == "" || $numberAttrib->value == "") {
            return $this->reRenderPage($page, tl::tl("Not all parameters were properly set"));
        }

        $ldapServer = null;
        if($page->ldapId === 0) {
            $ldapServer = new LdapServer();
        }
        if($ldapServer == null && isset($reportingSession->ldapServer)) {
            $ldapServer = $reportingSession->ldapServer;
        }
        if($ldapServer == null) {
            return $this->reRenderPage($page, tl::tlx('LDAP server {$arg[1]} deleted in the meantime.', $pbx->value));
        }
        
        $ldapServer->active = (isset($state->value["on"]) && $state->value["on"] === true) ? true : false;
        $ldapServer->pbx = $pbx->value;
        $ldapServer->host = $host->value;
        $ldapServer->port = $port->value;
        $ldapServer->password = $password->value;
        $ldapServer->user = $user->value;
        $ldapServer->base = $base->value;
        $ldapServer->filter = $filter->value;
        $ldapServer->nameAttrib = $nameAttrib->value;
        $ldapServer->numberAttrib = $numberAttrib->value;
        $ldapServer->cc = $cc->value;
        $ldapServer->ac = $ac->value;
        $ldapServer->np = $np->value;
        $ldapServer->ip = $ip->value;
        $ldapServer->el = $el->value;
        $ldapServer->sn = $sn->value;
        if(isset($tls->value["on"]) && $tls->value["on"] === true)
            $ldapServer->server = "ldaps://".$ldapServer->host.":".$ldapServer->port;
        else
            $ldapServer->server = "ldap://".$ldapServer->host.":".$ldapServer->port;
        $oldPbx = $page->ldapPbx;
        $rollback = false;
        if(self::insertLdapServer($ldapServer, $oldPbx, $error, $rollback) === false) {
            return $this->reRenderPage($page, $error, $rollback);
        }
        
        $ldapId = $page->getField("ldap_id");
        $ldapId->value = $page->ldapId = 0;
        $arg = "";
        if($oldPbx == "") {
            $arg = "&createdldap=".urlencode($pbx->value);
        }
        else {
            $arg = "&editedldap=".urlencode($pbx->value);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs().$arg);
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
    }

    function insertLdapServer(LdapServer &$ldapServer, $oldName, &$error, &$rollback) {
        $rollback = false;
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        $query = "LOCK TABLE ldap_servers IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        
        $insertId = "";
        $idStr = "";
        if($ldapServer->id != 0) {
            $insertId = $ldapServer->id.", ";
            $idStr = "id, ";
            if(ButtonDeleteLdap::delete($ldapServer->id) === false) {
                $rollback = true;
                $error = pg_last_error($reportingDb->db);
                return false;
            }
        }
        
        $query = "INSERT INTO ldap_servers ({$idStr}server, host, port, password, username, base, filter, number_attrib, name_attrib, pbx, cc, ac, np, ip, el, sn, active) VALUES (".
            $insertId.
            "'".pg_escape_string($ldapServer->server)."', ".
            "'".pg_escape_string($ldapServer->host)."', ".
            "".pg_escape_string($ldapServer->port).", ".
            "'".pg_escape_string(rc4_encrypt(strrev($ldapServer->host), $ldapServer->password))."', ".
            "'".pg_escape_string($ldapServer->user)."', ".
            "'".pg_escape_string($ldapServer->base)."', ".
            "'".pg_escape_string($ldapServer->filter)."', ".
            "'".pg_escape_string($ldapServer->numberAttrib)."', ".
            "'".pg_escape_string($ldapServer->nameAttrib)."', ".
            "'".pg_escape_string($ldapServer->pbx)."', ".
            "'".pg_escape_string($ldapServer->cc)."', ".
            "'".pg_escape_string($ldapServer->ac)."', ".
            "'".pg_escape_string($ldapServer->np)."', ".
            "'".pg_escape_string($ldapServer->ip)."', ".
            "'".pg_escape_string($ldapServer->el)."', ".
            "'".pg_escape_string($ldapServer->sn)."', ".
            "'".($ldapServer->active ? "1" : "0")."'".
            ") RETURNING id";
        if(($result = $reportingDb->run($query)) === false) {
            $error = pg_last_error($reportingDb->db);
            $rollback = true;
            return false;
        }
        $idRow = pg_fetch_object($result);
        $id = $idRow->id;
        $ldapServer->id = $id;
        if(file_put_contents(Config::ldapServersChanged, "") === false) {
            $error = tl::tl("Couldn't write ldap state file!");
            $rollback = true;
            return false;
        }
        if($reportingDb->run("COMMIT") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }               
        return true;
    }

    function reRenderPage(InputPage &$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}
    
class ButtonMailConfig extends MainInputPageOkCancelButton {
    public function ButtonMailConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $mail = null;
        $error = "";
        if($page->mailId === 0) {
            $mail = new ReportMail();
        }
        else {
            $mail = $reportingSession->reportMail;
        }
        if($mail == null) {
            return $this->reRenderPage($page, tl::tlx('Report mail {$arg[1]} deleted in the meantime.', $page->mailName));
        }
        if($reportingSession->isUser() && $mail->id > 0 && $mail->userId != $reportingSession->user->id) {
            return $this->reRenderPage($page, tl::tlx('You have no rights to edit the report mail {$arg[1]}!', $page->mailName));
        }
        
        $mailName = $page->getField("mailname");
        $addresses = $page->getField("addresses");
        $pbx = $page->getField(ReportingInputPage::valPbx);
        $pbxObject = $page->getField(ReportingInputPage::valObject);
        $list = $page->getField(ReportingInputPage::valList);
        $groupBy = $page->getField(ReportingInputPage::valGroupBy);
        $sortBy = $page->getField(ReportingInputPage::valSortBy);
        $sortOrder = $page->getField(ReportingInputPage::valSortOrder);
        $hiddenDigits = $page->getField(ReportingInputPage::valHideDigits);
        $format = $page->getField("outputformat");
        $interval = $page->getField("interval");
        $dayOfWeek = $page->getField("dayofweek");
        $dayOfMonth = $page->getField("dayofmonth");
        $time = $page->getField("time");
        $lang = $page->getField("maillang");
        $compress = $page->getField("compress");
        $dispDuration = $page->getField(ReportingInputPage::valDisplayedDuration);
        $anonym = $page->getField(ReportingInputPage::valAnonym);
        $bulk = $page->getField(ReportingInputPage::valBulk);
        $sender = $page->getField(ReportingInputPage::valSender);
        $bulkVal = ($bulk->value["on"] ? true : false);
                
        if(!$mailName->valid() || (!$bulkVal && !$addresses->valid()) || !$time->valid() ||
            ($sender->value != "" && !$sender->valid())) {
            return $this->reRenderPage($page, "");
        }
        $timeVals = explode(":", $time->value);
        if($this->checkMailName($mailName->value, $page->mailId, $error) === false) {
            return $this->reRenderPage($page, $error);
        }
        $mail->name = $mailName->value;
        $mail->pbx = array_search(true, $pbx->value);
        $mail->mails = $addresses->value;
        $mail->pbxObject = $pbxObject->value;
        $mail->groupBy = array_search(true, $groupBy->value);
        $mail->sortBy = array_search(true, $sortBy->value);
        $tmpSortOrder = array_search(true, $sortOrder->value);
        $mail->sortOrder = (($tmpSortOrder == "on" && $sortOrder->value["on"]) ? "ASC" : "DESC");
        $mail->hideDigits = $hiddenDigits->value;
        $mail->interval = intval(array_search(true, $interval->value));
        $mail->format = array_search(true, $format->value);
        $mail->dayOfMonth = array_search(true, $dayOfMonth->value);
        $mail->dayOfWeek = array_search(true, $dayOfWeek->value);
        $mail->timeHour = $timeVals[0];
        $mail->timeMinute = $timeVals[1];
        $mail->lang = array_search("true", $lang->value);
        $mail->filters = array();
        $mail->compress = ($compress->value["on"] ? true : false);
        $mail->displayedDuration = array_search(true, $dispDuration->value);
        $mail->anonym = ($anonym->value["on"] ? true : false);
        $mail->bulk = $bulkVal;
        $mail->sender = $sender->value;
        $filters = $page->getFilterListArray();
        foreach($list->value as $value) {
            $filter = array_search(true, $value[$list->name."select"]);
            if($filter != "") {
                $mail->filters[$filter] = $filters[$filter];
            }
        }

        if($pbxObject->value == "" && count($mail->filters) == 0) {
            return $this->reRenderPage($page, tl::tl("You have to select a filter or set a PBX object!"));
        }
        $rollback = false;
        if($this->insertReportMail($mail, $page->mailName, $error, $rollback) === false) {
            return $this->reRenderPage($page, $error, $rollback);
        }
        
        $reportingSession->updateReportMail($mail);
        $oldName = $page->mailName;
        $mailId = $page->getField("mail_id");
        $mailId->value = $page->mailId = 0;
        $arg = "";
        if($oldName == "") {
            $arg = "&createdmail=".urlencode($mail->name);
        }
        else {
            $arg = "&editedmail=".urlencode($mail->name);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs().$arg);
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
    }

    function checkMailName($name, $id, &$error) {
        $reportingDb = get_reporting_db();
        $query = "SELECT id FROM mails WHERE name ILIKE '".pg_escape_string($name)."' AND (id != $id OR $id=0)";
        $result = $reportingDb->run($query);
        if(($result !== false) && (pg_num_rows($result) > 0)) {
            $error = tl::tl("Report Mail name already exists!");
            return false;
        }
        return true;
    }
       
    function insertReportMail(ReportMail &$mail, $oldName, &$error, &$rollback) {
        $rollback = false;
        global $reportingSession;
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }       
        $query = "LOCK TABLE mails, mail_filters IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        
        $insertId = "";
        $idStr = "";
        if($mail->id != 0) {
            $insertId = $mail->id.", ";
            $idStr = "id, ";
            if(ButtonDeleteMail::delete($mail->id, "", $error) === false) {
                $rollback = true;
                return false;
            }
        }
        else {
            if($this->checkMailName($mail->name, $mail->id, $error) === false) {
                return false;
            }
        }        
        
        $query = "INSERT INTO mails ({$idStr}name, pbx, pbx_object, group_by, interval, mails, format, hide_digits, time, day_of_week, day_of_month, sort_by, sort_order, lang, user_id, compress, displayed_duration, anonym, bulk, sender) VALUES (".
            $insertId.
            "'".pg_escape_string($mail->name)."', ".
            "'".pg_escape_string($mail->pbx)."', ".
            "'".pg_escape_string($mail->pbxObject)."', ".
            "'".pg_escape_string($mail->groupBy)."', ".
            $mail->interval.", ".
            "'".pg_escape_string($mail->mails)."', ".
            "'".pg_escape_string($mail->format)."', ".
            $mail->hideDigits.", ".
            "'".pg_escape_string($mail->getTime())."', ".
            $mail->dayOfWeek.", ".
            $mail->dayOfMonth.", ".
            "'".pg_escape_string($mail->sortBy)."', ".
            "'".pg_escape_string($mail->sortOrder)."', ".
            "'".pg_escape_string($mail->lang)."', ".
            ($reportingSession->isUser() ? $reportingSession->user->id : "NULL").", ".
            ($mail->compress ? "true" : "false").", ".
            "'".pg_escape_string($mail->displayedDuration)."', ".
            ($mail->anonym ? "true" : "false").", ".
            ($mail->bulk ? "true" : "false").", ".
            "'".pg_escape_string($mail->sender)."' ".
        ") RETURNING id";
        if(($result = $reportingDb->run($query)) === false) {
            $error = pg_last_error($reportingDb->db);
            $rollback = true;
            return false;
        }
        $idRow = pg_fetch_object($result);
        $id = $idRow->id;
        $mail->id = $id;
        $query = "";
        foreach($mail->filters as $filterId => $filterName) {
            $query .= "INSERT INTO mail_filters (mail_id, filter_id) VALUES (".
            "$id, ".
            $filterId.
            ");";
        }
        if($query != "" && $reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        
        $cron = new CronJobs(Config::cronReportingMails);
        $cronName = $cron::encodePHPParameters($mail->name);
        $cron->remove($cron::encodePHPParameters($oldName));
        $job = new CronJob();
        $job->hour = $mail->timeHour;
        $job->min = $mail->timeMinute;
        $job->dayOfMonth = "*";
        $job->month = "*";
        $job->dayOfWeek = "*";
        if($mail->interval == 1) {      // weekly
            $job->dayOfWeek = $mail->dayOfWeek;
        }
        elseif($mail->interval == 2) {  // monthly
            $job->month = "1-12";
            $job->dayOfMonth = $mail->dayOfMonth;
        }
        
        $job->cmd = "www-data [ -x /usr/bin/php5-cgi ] && nice -1 /usr/bin/php5-cgi ".Config::webRoot."/".Config::applicationsDir."/innovaphone-reporting/report.php \"remote=1&mail=$cronName\" >> ".Config::reportingLog." 2>&1";
        $cron->add($cronName, $job);
        if($cron->submit(Config::cronReportingMails) === false) {
            $error = $cron->error;
            return false;
        }
            
        if($reportingDb->run("COMMIT") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }               
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}    

class ButtonUserConfig extends MainInputPageOkCancelButton {
    public function ButtonUserConfig($name, $pageNo, InputPage &$page, $nextMode = null, $prevMode = null) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $user = null;
        $error = "";
        if($page->userId === 0) {
            $user = new ReportingUser();
        }
        else {
            $user = $reportingSession->user;
        }
        if($user == null) {
            return $this->reRenderPage($page, tl::tlx('User {$arg[1]} deleted in the meantime.', $page->userName));
        }
        $userName = $page->getField("username");
        $password = $page->getField("password");
        $list = $page->getField(ReportingInputPage::valList);
        
        if(!$userName->valid() || ($user->id == 0 && !$password->valid()) || ($user->id > 0 && $password->value != "" && !$password->valid())) {
            return $this->reRenderPage($page, "");
        }
        if($this->checkUserName($userName->value, $page->userId, $error) === false) {
            $this->reRenderPage($page, $error);
        }
        $user->name = $userName->value;
        $user->password = $password->value;
        $user->filters = array();
        $filters = $page->getFilterListArray(true);
        foreach($list->value as $value) {
            $filter = array_search(true, $value[$list->name."select"]);
            if($filter > 0) {
                $user->filters[$filter] = $filters[$filter];
            }
        }
        if(count($user->filters) == 0) {
            return $this->reRenderPage($page, tl::tl("You have to select at least one base filter!"));
        }
        $rollback = false;
        if($this->insertUser($user, $page->userName, $error, $rollback) === false) {
            return $this->reRenderPage($page, $error, $rollback);
        }
        
        $reportingSession->updateUser($user);
        $oldName = $page->userName;
        $userId = $page->getField("user_id");
        $userId->value = $page->userId = 0;
        $arg = "";
        if($oldName == "") {
            $arg = "&createduser=".urlencode($user->name);
        }
        else {
            $arg = "&editeduser=".urlencode($user->name);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs().$arg);
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
    }

    function checkUserName($name, $id, &$error) {
        $reportingDb = get_reporting_db();
        $query = "SELECT id FROM users WHERE name ILIKE '".pg_escape_string($name)."' AND (id != $id OR $id=0)";
        $result = $reportingDb->run($query);
        if(($result !== false) && (pg_num_rows($result) > 0)) {
            $error = tl::tl("User name already exists!");
            return false;
        }
        return true;
    }
       
    function insertUser(ReportingUser &$user, $oldName, &$error, &$rollback) {
        $rollback = false;
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }       
        $query = "LOCK TABLE users, user_filters, filters IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        
        $query = "";
        $nameVal = "'".pg_escape_string($user->name)."'";
        $pwVal = "'".ReportingMethods::getMd5Pw($user->name, $user->password)."'";
        if($user->id != 0) {
            if(ButtonDeleteUser::delete($user->id, $user->filters, $error) === false) {
                $rollback = true;
                return false;
            }
            $query = "UPDATE users SET name=$nameVal"; 
            if($user->password != "") {
                $query .= ", password=$pwVal";
            }
            $query .= " WHERE id={$user->id}";
        }
        else {
            if($this->checkUserName($user->name, $user->id, $error) === false) {
                return false;
            }
            $query = "INSERT INTO users (name, password) VALUES ($nameVal, $pwVal)
                        RETURNING id;";
        }
        
        if(($result = $reportingDb->run($query)) === false) {
            $error = pg_last_error($reportingDb->db);
            $rollback = true;
            return false;
        }
        if($user->id == 0) {
            $idRow = pg_fetch_object($result);
            $user->id = $idRow->id;
        }
        $query = "INSERT INTO user_filters (user_id, filter_id) VALUES ";
        foreach($user->filters as $filterId => $filter) {
            $query .= "({$user->id}, $filterId),";
        }
        $query = rtrim($query, ",");
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            $rollback = true;
            return false;
        }
        if($reportingDb->run("COMMIT") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }               
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}    

class ButtonVoicePlayerConfig extends InputPageButtonOnlyAction {
    public function ButtonVoicePlayerConfig($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $reportingSession ReportingSession */global $reportingSession;
        $port = $page->getField("playerport");
        $reportingSession->reportingCfg->voicePlayer->port = $port->value;
        if($reportingSession->reportingCfg->write() === false) {
            return $this->reRenderPage($page, $reportingSession->reportingCfg->error);
        }
        $page->render();
    }
    
    function reRenderPage(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonPbxConfig extends InputPageButtonOnlyAction {
    public function ButtonPbxConfig($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        /* @var $reportingSession ReportingSession */global $reportingSession;
        $ident = $page->getField("ident");
        $reportingSession->reportingCfg->pbx->identifier = array_search(true, $ident->value);
        if($reportingSession->reportingCfg->write() === false) {
            return $this->reRenderPage($page, $reportingSession->reportingCfg->error);
        }
        $page->render();
    }
    
    function reRenderPage(InputPage &$page, $error) {
        $this->msg = $error;
        $page->render();
        return false;
    }
}

class ButtonReplicationConfig extends InputPageButtonOnlyAction {
    public function ButtonReplicationConfig($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = false;
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $drop = $page->getField("replmode");
        $server = $page->getField("server");
        $password = $page->getField("password");
        $interval = $page->getField("interval");
        $error = "";
        
        $oldServer = $reportingSession->reportingCfg->replication->server;
        $mode = array_search(true, $drop->value);
        if($mode != Replication::modeOff) {
            if(!$server->valid() || !$password->valid() || !$interval->valid()) {
                return $this->reRenderPage($page, "");
            }    
        }
        if($mode == Replication::modeOff) {
            $reportingSession->reportingCfg->replication = new Replication();
        }
        else {
            $reportingSession->reportingCfg->replication->mode = $mode;
            $reportingSession->reportingCfg->replication->server = $server->value;
            $reportingSession->reportingCfg->replication->password = $password->value;
            $reportingSession->reportingCfg->replication->interval = $interval->value;
        }
        
        if($reportingSession->reportingCfg->write() === false) {
            return $this->reRenderPage($page, $reportingSession->reportingCfg->error);
        }
        
        $addServer = false;
        $ips = read_pg_ips($error);
        $oldIpExists = array_search($oldServer, $ips);
        $newIpExists = array_search($server->value, $ips);
        
        if($oldIpExists !== false && ($oldServer != $server->value || $mode == Replication::modeOff)) {
            if(ButtonAddDbIp::ip($oldServer, false, "innovaphone-reporting", $error) === false) {
                return $this->reRenderPage($page, $error);
            }
            unset($ips[$oldIpExists]);
            $oldIpExists = false;
        }
    
        $cron = new CronJobs(Config::cronReportingReplication);
        if($reportingSession->reportingCfg->replication->mode == Replication::modeOff) {
            $cron->remove(Config::reportingReplicationBin);
            $addServer = false;
        }
        else {
            $job = new CronJob();
            if((int)$interval->value == 60) {
                $job->hour = "*/1";
                $job->min = "0";
            }
            else {
                $job->min = "*/{$interval->value}";
            }
            $job->cmd = "root [ -x /usr/bin/sudo ] && /usr/bin/sudo -u innovaphone-reporting nice -1 ".Config::reportingReplicationSh." \"".Config::reportingReplicationBinLock."\" \"".Config::reportingReplicationBin."\" \"".Config::reportingReplicationStatus."\" >> ".Config::reportingReplicationLog." 2>&1";
            $cron->add(Config::reportingReplicationBin, $job);
        }
        $cron->submit(Config::cronReportingReplication);
        if($cron->error != "") {
            return $this->reRenderPage($page, $cron->error);
        }
        if($mode != Replication::modeOff && $newIpExists === false) {
            if(ButtonAddDbIp::ip($reportingSession->reportingCfg->replication->server, true, "innovaphone-reporting", $error) === false) {
                return $this->reRenderPage($page, $error);
            }
        }
        
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&replchanged=1");
    }
    
    function reRenderPage(InputPage &$page, $error) {
        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonAskDeleteImage extends InputPageAction {
    public function ButtonAskDeleteImage($name, $value) {
        parent::__construct($name, $value);
        $this->autorender = false;
    }

    function action(InputPage &$page, $phase) {
        $listImages = $page->getField("listimages");
        $listImagesArray = $listImages->value;
        foreach($listImagesArray as $key => $value) {
            $page->imageName = $key;
            $this->parent->imageName = $key;
            break;
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ReportingConfigConsts::modeDeleteImage);
    }
}
class ButtonAskDeleteFilter extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteFilter($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ReportingConfigConsts::modeDeleteFilter);
    }
}

class ButtonAskDeleteLdap extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteLdap($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ReportingConfigConsts::modeDeleteLdap);
    }
}

class ButtonDeleteLdap extends MainInputPageOkCancelButton {
    public function ButtonDeleteLdap($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }
        $query = "LOCK TABLE ldap_servers IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db), true);
        }
        if(self::delete($page->ldapId) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db), true);
        }
        if(file_put_contents(Config::ldapServersChanged, "") === false) {
            return $this->reRenderPage($page, tl::tl("Couldn't write ldap state file!"), true);
        }
        if($reportingDb->run("COMMIT") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }       
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&tab=".ReportingConfigConsts::tabLdap."&ldap_id=0&ldapdeleted=".urlencode($page->ldapPbx));
    }
    
    static function delete($id) {
        $reportingDb = get_reporting_db();
        $query = "DELETE FROM ldap_servers WHERE id=$id;";
        
        if($reportingDb->run($query) === false) {
            return false;
        }
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }

        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonAskDeleteCDRs extends MainInputPageOkCancelButton {
    public function ButtonAskDeleteCDRs($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        $from = $page->getField("from");
        $to = $page->getField("to");
        if(($from->value + 86400) > 2147483648 || $to->value > 2147483648) {
            $this->msg = tl::tl("Make sure you select a year below 2038 (32-bit problem)");
            $page->render();
            exit;
        }
        if($from->value > $to->value) {
            $this->msg = tl::tlx('From date {$arg[1]} must be lower or equal to to date  {$arg[2]}.', InputPageDateField::translateDate($from->value), InputPageDateField::translateDate($to->value));
            $page->render();
            exit;
        }
        $mode = $page->getField("mode");
        $mode->value = $page->currentMode = PostgresPage::modeDeleteCdrs;
        $page->reload();
        //$page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".PostgresPage::modeDeleteCdrs);
    }
}

class ButtonConfigureCleanup extends MainInputPageOkCancelButton {
    public function ButtonConfigureCleanup($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $active = $page->getField("active");
        $months = $page->getField("months");
        $weeks = $page->getField("weeks");
        $weeksVal = intval($weeks->value);
        $monthsVal = intval($months->value);
        if($monthsVal < 0 || $weeksVal < 0) {
            $this->msg = tl::tl("Enter 0 or positive numbers!");
            return $page->render();
        }
        if($weeksVal == 0 && $monthsVal == 0) {
            $this->msg = tl::tl("Either week or month must have a non zero value!");
            return $page->render();
        }
        $cron = new CronJobs(Config::cronReportingCDRCleanup);
        $cronName = "cdr_cleanup.php";
        $cron->remove($cronName);
        if($active->value["on"]) {
            $job = new CronJob();
            $job->hour = "2";
            $job->min = "0";
            $job->dayOfMonth = "*";
            $job->month = "*";
            $job->dayOfWeek = "*";

            $job->cmd = "root [ -x /usr/bin/php5-cgi ] && nice -1 /usr/bin/php5-cgi -q ".Config::webRoot."/".Config::applicationsDir."/innovaphone-reporting/$cronName \"months={$monthsVal}&weeks={$weeksVal}\" >> ".Config::reportingCDRCleanupLog." 2>&1";
            $cron->add($cronName, $job);
        }
        if($cron->submit(Config::cronReportingCDRCleanup) === false) {
            $this->msg = $cron->error;
            return $page->render();
        }
        $reportingSession->reportingCfg->cleanup->active = ($active->value["on"] ? true : false);
        $reportingSession->reportingCfg->cleanup->months = $monthsVal;
        $reportingSession->reportingCfg->cleanup->weeks = $weeksVal;
        if($reportingSession->reportingCfg->write() === false) {
            $this->msg = $reportingSession->reportingCfg->error;
            return $page->render();
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&cleanupconf=1");
    }
}

class ButtonDeleteCDRs extends MainInputPageOkCancelButton {
    public function ButtonDeleteCDRs($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        $reportingDb = get_reporting_db();
        $from = $page->getField("from");
        $to = $page->getField("to");
        $pbx = $page->getField(ReportingInputPage::valPbx);
        $pbxObject = $page->getField(ReportingInputPage::valObject);
        $cond = self::QueryConditions($pbxObject->value, array_search(true, $pbx->value), $from->value, $to->value);
        $error = "";
        $cdrs = 0;
        if(ReportingMethods::deleteCDRs($cond, $error, $cdrs) === false) {
            return $this->reRenderPage($page, $error);
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&deletedcdrs=$cdrs");
    }
    
    function reRenderPage(InputPage &$page, $error = "", $rollback = false) {
        $text = $page->getField("text");
        $text->autorender = false;
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }
        $this->msg = $error;
        $page->render();
        return false;
    }
    
    static function QueryConditions($pbxObject, $pbx, $from, $to) {
        global $reportingSession;
        
        return "cu.cn ILIKE '".pg_escape_string($pbxObject)."' AND 
                cu.{$reportingSession->reportingCfg->pbx->identifier}='".pg_escape_string($pbx)."' AND 
                cdrs.local_stamp >= '".date('Y-m-d', $from)."' AND cdrs.local_stamp < '".date('Y-m-d', ($to + 86400))."'";
    }
}

class ButtonAskDeleteMail extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteMail($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ReportingConfigConsts::modeDeleteMail);
    }
}

class ButtonDeleteMail extends MainInputPageOkCancelButton {
    public function ButtonDeleteMail($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $error = "";
        $mailReport = ReportingMethods::getReportMail($page->mailId, "", $error);
        if($reportingSession->isUser() && $mailReport->userId != $reportingSession->user->id) {
            return $this->reRenderPage($page, tl::tlx('You have no rights to delete the report mail {$arg[1]}!', $page->mailName));
        }
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }
        $query = "LOCK TABLE mail_filters, mails IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db), true);
        }
        if(self::delete($page->mailId, $page->mailName, $error) === false) {
            return $this->reRenderPage($page, $error, true);
        }
        
        if($reportingDb->run("COMMIT") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }       
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&tab=".ReportingConfigConsts::tabMail."&mail_id=0&deletedmail=".urlencode($page->mailName));
    }
    
    static function delete($id, $name, &$error) {
        if($name != "") {
            $cron = new CronJobs(Config::cronReportingMails);
            $cron->remove($cron::encodePHPParameters($name));
            if($cron->submit(Config::cronReportingMails) === false) {
                $error = $cron->error;
                return false;
            }
        }
        $reportingDb = get_reporting_db();
        $query = "DELETE FROM mail_filters WHERE mail_id=$id;";
        $query .= "DELETE FROM mails WHERE id=$id;";
        
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }

        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonAskDeleteUser extends InputPageButtonOnlyAction {
    public function ButtonAskDeleteUser($name, $value) {
        parent::__construct($name, $value);
        $this->anyway = true;
    }
    
    function action(InputPage &$page, $phase) {
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs()."&mode=".ReportingConfigConsts::modeDeleteUser);
    }
}

class ButtonDeleteUser extends MainInputPageOkCancelButton {
    public function ButtonDeleteUser($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }
        $query = "LOCK TABLE users, filters, user_filters, filter_conditions, filter_times, mails, mail_filters IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db), true);
        }
        $error = "";
        if(self::delete($page->userId, null, $error) === false) {
            return $this->reRenderPage($page, $error, true);
        }
        if($reportingDb->run("COMMIT") === false) {
            $this->reRenderPage($page, pg_last_error($reportingDb->db));
            return;
        }       
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&tab=".ReportingConfigConsts::tabUsers."&user_id=0&deleteduser=".urlencode($page->userName));
    }
    
    static function delete($id, $filters, &$error) {
        $reportingDb = get_reporting_db();
        $filterIds = "0";
        if($filters != null) {
            foreach($filters as $filterId => $filterName) {
                $filterIds .= ",$filterId";
            }
        }
        // delete user created filters with base filters, which do not belong to the user any more
        $query = "SELECT f.id 
                    FROM filters f
                    JOIN user_filters uf ON f.id=uf.filter_id
                    WHERE f.base_filter IS NOT NULL AND uf.user_id=$id AND f.base_filter NOT IN ($filterIds)";
        $result = $reportingDb->run($query);
        if($result === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        while(($row = pg_fetch_object($result))) {
            if(ButtonDeleteFilter::delete($row->id) === false) {
                $error = pg_last_error($reportingDb->db);
                return false;
            }
        }
        // delete user created mail reports
        $mailConds = "";
        if($filters != null) {
            // just delete mail reports with filters, which doesn't exist any more
            $mailConds = "AND (mf.filter_id IS NOT NULL OR mf.filter_id NOT IN ($filterIds)) AND (f.base_filter IS NOT NULL OR f.base_filter NOT IN ($filterIds))";
        }
        $query = "SELECT m.id, m.name 
                    FROM mails m
                    LEFT OUTER JOIN mail_filters mf ON m.id=mf.mail_id
                    LEFT OUTER JOIN filters f ON mf.filter_id=f.base_filter
                    WHERE m.user_id=$id $mailConds";
        $result = $reportingDb->run($query);
        if($result === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        while(($row = pg_fetch_object($result))) {
            if(ButtonDeleteMail::delete($row->id, $row->name, $error) === false) {
                return false;
            }
        }
        // delete user filter assignments
        $query = "DELETE FROM user_filters WHERE user_id=$id AND filter_id NOT IN (SELECT id FROM filters WHERE base_filter IN ($filterIds));";
        if($filters === null) {
            $query .= "DELETE FROM users WHERE id=$id;";
        }
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }

        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonDeleteFilter extends MainInputPageOkCancelButton {
    public function ButtonDeleteFilter($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        if(!ReportingMethods::checkUserFilter($page->filterId)) {
            return $this->reRenderPage($page, tl::tlx('You have no rights to delete the filter {$arg[1]}!', $page->filterName));
        }
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db));
        }
        $query = "LOCK TABLE filters, filter_times, filter_conditions, mail_filters, mails, users, user_filters IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db));
        }
        $query = "SELECT m.name AS name FROM mail_filters mf
                    JOIN mails m ON mf.mail_id=m.id
                    WHERE mf.filter_id={$page->filterId} OR mf.filter_id IN (
                        SELECT f.id FROM filters f WHERE f.base_filter={$page->filterId}
                        )";
        $result = $reportingDb->run($query);
        if($row = pg_fetch_object($result)) {
            return $this->reRenderPage($page, tl::tlx('This filter can\'t be deleted, as it is used in the report mail {$arg[1]}.', "'{$row->name}'"), true);
        }
        if(!$reportingSession->isUser()) {
            $query = "SELECT u.name AS name FROM user_filters uf
                        JOIN users u ON uf.user_id=u.id
                        JOIN filters f ON f.id=uf.filter_id
                        WHERE uf.filter_id={$page->filterId} AND f.base_filter IS NULL";
            $result = $reportingDb->run($query);
            if($row = pg_fetch_object($result)) {
                return $this->reRenderPage($page, tl::tlx('This filter can\'t be deleted, as it is used as base filter by user {$arg[1]}.', "'{$row->name}'"), true);
            }
        }
        if(self::delete($page->filterId) === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db), true);
        }
        if($reportingDb->run("COMMIT") === false) {
            return $this->reRenderPage($page, pg_last_error($reportingDb->db));
        }       
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&tab=".ReportingConfigConsts::tabFilter."&filter_id=0&deleted=".urlencode($page->filterName));
    }
    
    static function delete($id, $update = false) {
        $reportingDb = get_reporting_db();
        $cond = " (SELECT id FROM filters WHERE base_filter=$id)";
        $query = "DELETE FROM filter_conditions WHERE filter_id=$id";
        if(!$update) $query .=  " OR filter_id IN $cond";
        $query .= ";DELETE FROM filter_times WHERE filter_id=$id";
        if(!$update) $query .= " OR filter_id IN $cond";
        $query .= ";";
        if(!$update) {
            $query .= "DELETE FROM user_filters WHERE filter_id=$id OR filter_id IN $cond;";
            $query .= "DELETE FROM filters WHERE id IN $cond OR id=$id;";
        }
        
        if($reportingDb->run($query) === false) {
            return false;
        }
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }

        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class ButtonDeleteImage extends MainInputPageOkCancelButton {
    public function ButtonDeleteImage($name, $pageNo, InputPage &$page, $nextMode, $prevMode) {
        parent::__construct($name, $pageNo, $page, $nextMode, $prevMode);
    }

    function action(InputPage &$page, $phase) {
        $fileName = str_replace(array(" ", "(", ")"), array("\ ", "\(", "\)"), $page->imageName);
        $cmd="rm -f ".Config::imagesDir."/".$fileName;
        $chkDel = main_system($cmd, $retval);
        if($chkDel != "" || $retval != "") {
            echo "Image ".$page->imageName." could not be deleted";
            return;
        }
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&tab=".ReportingConfigConsts::tabImages."&deletedimage=".urlencode($page->imageName));
    }
}

class ButtonFilterConfig extends MainInputPageOkCancelButton {    
    public function ButtonFilterConfig($name, $pageNo, InputPage &$page) {
        parent::__construct($name, $pageNo, $page);
    }
    
    function action(InputPage &$page, $phase) {
        global $reportingSession;
        $filter = null;
        $error = "";
        if($page->filterId === 0) {
            $filter = new Filter();
        }
        else {
            $filter = $reportingSession->filter;
        }
        if($filter == null) {
            return $this->reRenderPage($page, tl::tlx('Filter {$arg[1]} deleted in the meantime.', $page->filterName));
        }
        if(!ReportingMethods::checkUserFilter($page->filterId)) {
            return $this->reRenderPage($page, tl::tlx('You have no rights to edit the filter {$arg[1]}!', $page->filterName));
        }
        $error = "";
        $filterName = $page->getField("filtername");
        $listLocal = $page->getField("listfilterlocalconds");
        $listRemote = $page->getField("listfilterremoteconds");
        $listPbx = $page->getField("listfilterpbxconds");
        $listTimes = $page->getField("listtimes");
        $fitlerDesc = $page->getField("filterdesc");
        //$callingState = $page->getField("callingstate");
        $callingDuration = $page->getField("callingduration");
        $callingDurationCompare = $page->getField("callingdurationcompare");
        $alertDuration = $page->getField("alertduration");
        $alertDurationCompare = $page->getField("alertdurationcompare");
        //$callDirection = $page->getField("calldirection");
        //$callType = $page->getField("calltype");
        $cStateAlert = $page->getField("callstatealert");
        $cStateConn = $page->getField("callstateconn");
        $cStateBusy = $page->getField("callstatebusy");
        $cStateNoChannel = $page->getField("callstatenochannel");
        $cDirIn = $page->getField("calldirin");
        $cDirOut = $page->getField("calldirout");
        $cDirCt = $page->getField("calldirct");
        $cDirCf = $page->getField("calldircf");
        $cTypeInt = $page->getField("calltypeint");
        $cTypeExt = $page->getField("calltypeext");
        $logoImage = $page->getField("listlogo");
        $baseFilter = $page->getField("basefilter");
        $anonym = $page->getField(ReportingInputPage::valAnonym);
        $logo = "";
        foreach($logoImage->value as $key => $val) {
            $logo = $key;
            break;
        }
        $footerImage = $page->getField("listfooter");
        $footer = "";
        foreach($footerImage->value as $key => $val) {
            $footer = $key;
            break;
        }
        if(!$filterName->valid()) {
            return $this->reRenderPage($page, "");
        }
        
        if($this->checkFilterName($filterName->value, $page->filterId, $error) === false) {
            return $this->reRenderPage($page, $error);
        }
        if(!$cStateAlert->value["on"] && !$cStateBusy->value["on"] && !$cStateConn->value["on"] && !$cStateNoChannel->value["on"]) {
            return $this->reRenderPage($page, tl::tl("Check at least one call state checkbox."));
        }
        if(!$cDirIn->value["on"] && !$cDirOut->value["on"] && !$cDirCt->value["on"] && !$cDirCf->value["on"]) {
            return $this->reRenderPage($page, tl::tl("Check at least one call direction checkbox."));
        }
        $conds = array();
        $rows = 0;
        if(!$this->getListConditions($listLocal, $conds, $rows, FilterCondition::typeLocal, $error) ||
            !$this->getListConditions($listRemote, $conds, $rows, FilterCondition::typeRemote, $error) ||
            !$this->getListConditions($listPbx, $conds, $rows, FilterCondition::typePbx, $error)) {
                return $this->reRenderPage($page, $error);
        }
        if(!count($conds)) {
            return $this->reRenderPage($page, tl::tl("Define at least one condition"));
        }
        $oldName = $filter->name;
        $filter->name = $filterName->value;
        $filter->conds = $conds;
        $filter->anonym = false;
        if(is_object($anonym) && $anonym->value["on"]) {
            $filter->anonym = true;
        }
        //$filter->callingState = array_search("true", $callingState->value);
        if($cStateAlert->value["on"] && $cStateBusy->value["on"] && $cStateConn->value["on"] && $cStateNoChannel->value["on"]) {
            $filter->callingState = ReportingConfigConsts::callingStateAll;
        }
        else {
            $filter->callingState = "";
            if($cStateAlert->value["on"]) {
                $filter->callingState .= "|".ReportingConfigConsts::callingStateAlert;
            }    
            if($cStateBusy->value["on"]) {
                $filter->callingState .= "|".ReportingConfigConsts::callingStateBusy;
            }    
            if($cStateConn->value["on"]) {
                $filter->callingState .= "|".ReportingConfigConsts::callingStateConn;
            }    
            if($cStateNoChannel->value["on"]) {
                $filter->callingState .= "|".ReportingConfigConsts::callingStateNoChannel;
            }
            $filter->callingState = ltrim($filter->callingState, "|");
        }
        $filter->callingDuration = $callingDuration->value;
        if($filter->callingDuration == "") {
            $filter->callingDurationCompare = ReportingConfigConsts::callingDurationJustGreater;
        }
        else {
            $filter->callingDurationCompare = array_search("true", $callingDurationCompare->value);
        }
        $filter->alertDuration = $alertDuration->value;
        if($filter->alertDuration == "") {
            $filter->alertDurationCompare = ReportingConfigConsts::callingDurationJustGreater;
        }
        else {
            $filter->alertDurationCompare = array_search("true", $alertDurationCompare->value);                
        }
        $filter->desc = $fitlerDesc->value;
        $filter->callType = "";
        if(is_object($cTypeInt) && is_object($cTypeExt)) {
            if($cTypeInt->value["on"] && $cTypeExt->value["on"]) {
                $filter->callType = ReportingConfigConsts::callTypeAll;
            }
            else {
                $filter->callType = "";
                if($cTypeInt->value["on"]) {
                    $filter->callType .= "|".ReportingConfigConsts::callTypeInt;
                }    
                if($cTypeExt->value["on"]) {
                    $filter->callType .= "|".ReportingConfigConsts::callTypeExt;
                }
                $filter->callType = ltrim($filter->callType, "|");
            }
        }
        //$filter->callDirection = array_search("true", $callDirection->value);
        if($cDirCf->value["on"] && $cDirCt->value["on"] && $cDirIn->value["on"] && $cDirOut->value["on"]) {
            $filter->callDirection = ReportingConfigConsts::callDirectionAll;
        }
        else {
            $filter->callDirection = "";
            if($cDirCf->value["on"]) {
                $filter->callDirection .= "|".ReportingConfigConsts::callDirectionCf;
            }    
            if($cDirCt->value["on"]) {
                $filter->callDirection .= "|".ReportingConfigConsts::callDirectionCt;
            }    
            if($cDirIn->value["on"]) {
                $filter->callDirection .= "|".ReportingConfigConsts::callDirectionIn;
            }    
            if($cDirOut->value["on"]) {
                $filter->callDirection .= "|".ReportingConfigConsts::callDirectionOut;
            }
            $filter->callDirection = ltrim($filter->callDirection, "|");
        }
        
        $filter->times = array();
        foreach($listTimes->value as $time) {
            if($time[$listTimes->name."from"] != "" && $time[$listTimes->name."to"] != "") {
                $filterTime = new FilterTime();
                $filterTime->setTime($time[$listTimes->name."from"], $time[$listTimes->name."to"]);
                $filterTime->day = array_search(true, $time[$listTimes->name."day"]);
                $filter->times[] = $filterTime;
            }
        }
        $filter->logo = $logo;
        $filter->footer = $footer;
        $filter->baseFilter = array_search("true", $baseFilter->value);
        
        $reportingDb = get_reporting_db();
        if($reportingSession->isUser()) {
            if($filter->baseFilter == 0) {
                return $this->reRenderPage($page, tl::tl("You have to assign a base filter!"));
            }
            $query = "SELECT COUNT(uf.filter_id) AS c
                        FROM user_filters uf
                        WHERE uf.user_id={$reportingSession->user->id} AND uf.filter_id={$filter->baseFilter}";
            $result = $reportingDb->run($query);
            $row = pg_fetch_object($result);
            if(intval($row->c) == 0) {
                return $this->reRenderPage($page, tl::tl("You are not allowed to use the selected base filter!"));
            }
        }
        
        // insert default columns
        $filter->columns = ReportingXmlCfg::getFilterColumns();
        
        $rollback = false;
        if($this->insertFilter($filter, $error, $rollback) === false) {
            return $this->reRenderPage($page, $error, $rollback);
        }
        
        $reportingSession->updateFilter($filter);

        $arg = "";
        if($page->filterId === 0) {
            $arg = "&createdfilter=".urlencode($filter->name);
        }
        else {
            $arg = "&editedfilter=".urlencode($filter->name);
        }
        $filterId = $page->getField("filter_id");
        $filterId->value = $page->filterId = 0;
        $page->warp($_SERVER["SCRIPT_NAME"]."?_page=".ReportingConfigConsts::pageConfig."&".$page->getArgs().$arg);
    }
    
    function checkFilterName($name, $id, &$error) {
        $reportingDb = get_reporting_db();
        $query = "SELECT id FROM filters WHERE name ILIKE '".pg_escape_string($name)."' AND (id != $id OR $id=0)";
        $result = $reportingDb->run($query);
        if(($result !== false) && (pg_num_rows($result) > 0)) {
            $error = tl::tl("Filter name already exists!");
            return false;
        }
        return true;
    }
    
    function getListConditions(MainInputPageListField &$list, &$conds, &$rows, $type, &$error) {
        $colNames = ReportingConfigConsts::getCondColNames();
        foreach($list->value as $val) {
            $value = $val[$list->name."value"];
            if($value != "") {
                $name = array_search("true", $val[$list->name."select"]);
                foreach($conds as $cond) {
                    if($cond->name == $name && $cond->value == $value) {
                        $error = tl::tlx('Condition {$arg[1]} is not unique.', "'{$colNames[$name]}:$value'");
                        return false;
                    }
                }            
                $conds[$rows++] = new FilterCondition($name, $value, $type);
            }
        }
        return true;
    }
    
    function insertFilter(Filter &$filter, &$error, &$rollback) {
        $rollback = false;
        global $reportingSession;
        $reportingDb = get_reporting_db();
        if($reportingDb->run("BEGIN") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }       
        $query = "LOCK TABLE filters, filter_times, filter_conditions, user_filters IN EXCLUSIVE MODE";
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        if($filter->baseFilter > 0) {
            $query = "SELECT id 
                        FROM filters 
                        WHERE id={$filter->baseFilter} AND base_filter IS NOT NULL";
            $result = $reportingDb->run($query);
            if(pg_num_rows($result) > 0) {
                $error = tl::tl("The base filter has a base filter itself and can't be used as base filter any more!");
                return false;
            }
            if($filter->id > 0) {
                $query = "SELECT id 
                            FROM filters
                            WHERE base_filter={$filter->id}";
                $result = $reportingDb->run($query);
                if(pg_num_rows($result) > 0) {
                    $error = tl::tl("You can't set a base filter, as the filter is used as base filter!");
                    return false;
                }
            }
        }
        
        $insertId = "";
        $idStr = "";
        if($filter->id != 0) {
            $insertId = $filter->id.", ";
            $idStr = "id, ";
            if(ButtonDeleteFilter::delete($filter->id, true) === false) {
                $rollback = true;
                $error = pg_last_error($reportingDb->db);
                return false;
            }
        }
        else {
            if($this->checkFilterName($filter->name, $filter->id, $error) === false) {
                return false;
            }
        }        
        $callDuration = "";
        if($filter->callingDuration == "") {
            $callDuration = "NULL";
        }
        else {
            $callDuration = $filter->callingDuration;
        }
        $alertDuration = "";
        if($filter->alertDuration == "") {
            $alertDuration = "NULL";
        }
        else {
            $alertDuration = $filter->alertDuration;
        }
        if($filter->id > 0) {
            $query = "UPDATE filters SET ";
        }
        else {
            $query = "INSERT INTO filters ";
        }
        $query .= "({$idStr}name, description, calling_state, calling_duration, calling_duration_compare, alert_duration, alert_duration_compare, call_direction, call_type, logo, footer, base_filter, anonym) ";
        if($filter->id > 0) {
            $query .= "=(";
        }
        else {
            $query .= "VALUES (";
        }
        $query.= $insertId.
            "'".pg_escape_string($filter->name)."', ".
            "'".pg_escape_string($filter->desc)."', ".
            "'{$filter->callingState}', ".
            "$callDuration, ".
            "'{$filter->callingDurationCompare}', ".
            "$alertDuration, ".
            "'{$filter->alertDurationCompare}', ".
            "'{$filter->callDirection}', ".
            "'{$filter->callType}', ".
            "'{$filter->logo}', ".
            "'{$filter->footer}', ".
            ($filter->baseFilter == 0 ? "NULL" : $filter->baseFilter).", ".
            ($filter->anonym ? "true" : "false").
            ") ";
        if($filter->id > 0) {
            $query .= "WHERE id={$filter->id} ";
        }
        $query .= "RETURNING id";
        if(($result = $reportingDb->run($query)) === false) {
            $error = pg_last_error($reportingDb->db);
            $rollback = true;
            return false;
        }
        $idRow = pg_fetch_object($result);
        $id = $idRow->id;
        $oldFilterId = $filter->id;
        $filter->id = $id;
        $query = "";
        foreach($filter->conds as $cond) {
            /* postgresql performs a regex comparision, need to convert % */
            if(!strcmp($cond->name, ReportingConfigConsts::colGroups)) {
                $newCondValue = ""; 
                if(substr($cond->value,0,1) == "%")
                    $newCondValue = ".*";
                else
                    $newCondValue = substr($cond->value,0,1);
                if(substr($cond->value,-1,1) == "%") {
                    $newCondValue .= substr($cond->value,1,-1);
                    $newCondValue .= ".*";
                } else
                    $newCondValue .= substr($cond->value,1);
                $cond->value = $newCondValue;
            }
            $query .= "INSERT INTO filter_conditions (filter_id, name, value, type) VALUES (".
            "$id, ".
            "'".pg_escape_string($cond->name)."', ".
            "'".pg_escape_string($cond->value)."', ".
            "'".pg_escape_string($cond->type)."');";
        }
        unset($col);
        foreach($filter->times as $time) {
            $query .= "INSERT INTO filter_times (filter_id, time_from, time_to, time_day) VALUES (".
            "$id, ".
            "'".pg_escape_string($time->getFrom())."', ".
            "'".pg_escape_string($time->getTo())."', ".
            pg_escape_string($time->day).
            ");";
        }
        if($reportingDb->run($query) === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }
        if($reportingSession->isUser() && $oldFilterId == 0) {
            $query = "INSERT INTO user_filters (user_id, filter_id) VALUES ({$reportingSession->user->id}, {$filter->id})";
            if($reportingDb->run($query) === false) {
                $error = pg_last_error($reportingDb->db);
                return false;
            }
        }
        if($reportingDb->run("COMMIT") === false) {
            $error = pg_last_error($reportingDb->db);
            return false;
        }               
        return true;
    }
    
    function reRenderPage(&$page, $error, $rollback = false) {
        if($rollback) {
            $reportingDb = get_reporting_db();
            $reportingDb->run("ROLLBACK");
        }

        $this->msg = $error;
        $page->pages[$page->pageNo]->fillLists();
        $page->render();
        return false;
    }
}

class PageConfig extends PostgresPage {
    public function getHelpAnchor() {
        switch($this->parent->currentTab) {
            case ReportingConfigConsts::tabFilter: 
                switch($this->parent->currentMode) {
                    case ReportingConfigConsts::modeFilter:
                        return "#Filters";
                    case ReportingConfigConsts::modeDeleteFilter:
                        return "#Filters";
                    default: return "#Filters";
                }
            case ReportingConfigConsts::tabPgsql: 
                switch($this->parent->currentMode) {
                    case PostgresPage::modeInRePgPw:
                        return "#Password";
                    case PostgresPage::modeAskDeleteCdrs:
                    case PostgresPage::modeDeleteCdrs:
                        return "#Delete_CDRs";
                    case PostgresPage::modeCDRCleanup:
                        return "#Configure_scheduled_CDR_cleanup";
                    default: return "#Database";
                }
            case ReportingConfigConsts::tabReplication:
                switch($this->parent->currentMode) {
                    default: return "#Replication";
                }
            case ReportingConfigConsts::tabLdap:
                switch($this->parent->currentMode) {
                    default: return "#Ldap";
                }
            case ReportingConfigConsts::tabMail:
                switch($this->parent->currentMode) {
                    default: return "#Report_Mails";
                }
            case ReportingConfigConsts::tabImages:
                switch($this->parent->currentMode) {
                    default: return "#Images";
                }
            case ReportingConfigConsts::tabUsers:
                switch($this->parent->currentMode) {
                    default: return "#Users";
                }
            case ReportingConfigConsts::tabPbx:
                switch($this->parent->currentMode) {
                    default: return "#PBX";
                }
            case ReportingConfigConsts::tabVoicePlayer:
                {
                    return "#Voice_Player";
                }
            default: return "";
        }
    }
    
    public function getDefaultTab() {
        return ReportingConfigConsts::tabFilter;
    }
    
    public function getHead() {
        return "";//($this->parent->filterId == 0 ? "" : htmlentities_utf8($this->parent->filterName));
    }
    
    public function createFields() {
        global $reportingSession;
        $this->addSection(ReportingConfigConsts::tabFilter);
        $this->addSection(ReportingConfigConsts::tabPbx);
        $this->addSection(ReportingConfigConsts::tabPgsql);
        $this->addSection(ReportingConfigConsts::tabReplication, 2);
        $this->addSection(ReportingConfigConsts::tabLdap);
        $this->addSection(ReportingConfigConsts::tabMail);
        $this->addSection(ReportingConfigConsts::tabImages);
        $this->addSection(ReportingConfigConsts::tabUsers);
        $this->addSection(ReportingConfigConsts::tabVoicePlayer);
        
        $genericHint = "";
        
        $this->parent->addTabMenuItem(tl::tl("Filter"), "javascript:submitTab('".ReportingConfigConsts::tabFilter."')", $this->parent->currentTab == ReportingConfigConsts::tabFilter);
        if(!$reportingSession->isUser()) {
            $this->parent->addTabMenuItem(tl::tl("PBX"), "javascript:submitTab('".ReportingConfigConsts::tabPbx."')", $this->parent->currentTab == ReportingConfigConsts::tabPbx);
            $this->parent->addTabMenuItem(tl::tl("Database"), "javascript:submitTab('".ReportingConfigConsts::tabPgsql."')", $this->parent->currentTab == ReportingConfigConsts::tabPgsql);
            $this->parent->addTabMenuItem(tl::tl("Replication"), "javascript:submitTab('".ReportingConfigConsts::tabReplication."')", $this->parent->currentTab == ReportingConfigConsts::tabReplication);
            $this->parent->addTabMenuItem(tl::tl("LDAP"), "javascript:submitTab('".ReportingConfigConsts::tabLdap."')", $this->parent->currentTab == ReportingConfigConsts::tabLdap);
            $this->parent->addTabMenuItem(tl::tl("Voice Player"), "javascript:submitTab('".ReportingConfigConsts::tabVoicePlayer."')", $this->parent->currentTab == ReportingConfigConsts::tabVoicePlayer);
        }
        $this->parent->addTabMenuItem(tl::tl("Report Mails"), "javascript:submitTab('".ReportingConfigConsts::tabMail."')", $this->parent->currentTab == ReportingConfigConsts::tabMail);
        if(!$reportingSession->isUser()) {
            $this->parent->addTabMenuItem(tl::tl("Images"), "javascript:submitTab('".ReportingConfigConsts::tabImages."')", $this->parent->currentTab == ReportingConfigConsts::tabImages);
            $this->parent->addTabMenuItem(tl::tl("Users"), "javascript:submitTab('".ReportingConfigConsts::tabUsers."')", $this->parent->currentTab == ReportingConfigConsts::tabUsers);
        }
        
        switch($this->parent->currentTab) {
            case ReportingConfigConsts::tabFilter: 
                $this->tabFilter();
                break;
            case ReportingConfigConsts::tabPbx: 
                $this->tabPbx();
                break;
            case ReportingConfigConsts::tabPgsql:
                $this->tabPgSql("innovaphone-reporting", ReportingConfigConsts::pageConfig, Replication::modeStandby);
                break;
            case ReportingConfigConsts::tabReplication:
                $this->tabReplication();
                break;
            case ReportingConfigConsts::tabLdap:
                $this->tabLdap();
                break;
            case ReportingConfigConsts::tabMail:
                $this->tabMail();
                break;
            case ReportingConfigConsts::tabImages:
                $this->tabImages();
                break;      
            case ReportingConfigConsts::tabUsers:
                $this->tabUsers();
                break;      
            case ReportingConfigConsts::tabVoicePlayer:
                $this->tabVoicePlayer();
                break;
        }
    }
    
    function addActionText() {
        $text = $this->addField(new InputPageText("filtertext_{$this->currentSection}" , ""));
        $text->autorender = false;
        $text->fullline = true;
        $createdFilter = InputPage::formInput("createdfilter");
        $editedFilter = InputPage::formInput("editedfilter");
        $deletedFilter = InputPage::formInput("deletedfilter");
        $changed = InputPage::formInput("pwchanged");
        $ipAdded = InputPage::formInput("ipadded");
        $ipDeleted = InputPage::formInput("ipdeleted");
        $ldapCreated = InputPage::formInput("createdldap");
        $ldapEdited = InputPage::formInput("editedldap");
        $ldapDeleted = InputPage::formInput("ldapdeleted");
        $mailDeleted = InputPage::formInput("deletedmail");
        $mailCreated = InputPage::formInput("createdmail");
        $mailEdited = InputPage::formInput("editedmail");
        $userDeleted = InputPage::formInput("deleteduser");
        $userCreated = InputPage::formInput("createduser");
        $userEdited = InputPage::formInput("editeduser");
        $replChanged = InputPage::formInput("replchanged");
        $cdrsDeleted = InputPage::formInput("deletedcdrs");
        $cdrCleanup = InputPage::formInput("cleanupconf");
        
        if(isset($createdFilter )) {
            $text->value = tl::tlx('Filter {$arg[1]} successfully created.', "<b>".htmlentities_utf8($createdFilter)."</b>");
        }
        elseif(isset($editedFilter )) {
            $text->value = tl::tlx('Filter {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($editedFilter)."</b>");
        }
        elseif(isset($deletedFilter)) {
            $text->value = tl::tlx('Filter {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($deletedFilter)."</b>");
        }
        elseif(isset($imageDeleted)) {
            $text->value = tl::tlx('Image {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($imageDeleted)."</b>");
        }
        elseif(isset($imageUploaded)) {
            $text->value = tl::tlx('Image {$arg[1]} successfully uploaded.', "<b>".htmlentities_utf8($imageUploaded)."</b>");
        }
        elseif(isset($changed)) {
            $text->value = tl::tl("Password successfully changed.");
        }
        elseif(isset($ipAdded)) {
            $text->value = tl::tl("IP successfully added.");
        }
        elseif(isset($ipDeleted)) {
            $text->value = tl::tl("IP(s) successfully deleted.");
        }
        elseif(isset($ldapCreated)) {
            $text->value = tl::tlx('LDAP server {$arg[1]} successfully created.', "<b>".htmlentities_utf8($ldapCreated)."</b>");
        }
        elseif(isset($ldapEdited)) {
            $text->value = tl::tlx('LDAP server {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($ldapEdited)."</b>");
        }
        elseif(isset($ldapDeleted)) {
            $text->value = tl::tlx('LDAP server {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($ldapDeleted)."</b>");
        }
        elseif(isset($mailCreated)) {
            $text->value = tl::tlx('Report mail {$arg[1]} successfully created.', "<b>".htmlentities_utf8($mailCreated)."</b>");
        }
        elseif(isset($mailEdited)) {
            $text->value = tl::tlx('Report mail {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($mailEdited)."</b>");
        }
        elseif(isset($mailDeleted)) {
            $text->value = tl::tlx('Report mail {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($mailDeleted)."</b>");
        }
        elseif(isset($userCreated)) {
            $text->value = tl::tlx('User {$arg[1]} successfully created.', "<b>".htmlentities_utf8($userCreated)."</b>");
        }
        elseif(isset($userEdited)) {
            $text->value = tl::tlx('User {$arg[1]} successfully edited.', "<b>".htmlentities_utf8($userEdited)."</b>");
        }
        elseif(isset($userDeleted)) {
            $text->value = tl::tlx('User {$arg[1]} successfully deleted.', "<b>".htmlentities_utf8($userDeleted)."</b>");
        }
        elseif(isset($replChanged)) {
            $text->value = tl::tl("Replication successfully changed.");
        }
        elseif(isset($cdrsDeleted)) {
            $text->value = tl::tlx('{$arg[1]} CDR(s) successfully deleted.', "<b>$cdrsDeleted</b>");
        }
        elseif(isset($cdrCleanup)) {
            $text->value = tl::tl("Scheduled CDR cleanup configured.");
        }
        
        if($text->value != "") {
            $text->autorender = true;
            $text->value = "<i>{$text->value}</i><p>&nbsp;</p>";
        }
    }
    
    public function tabVoicePlayer() {
        $this->currentSection = $this->parent->currentTab;
        /* @var $reportingSession ReportingSession */global $reportingSession;
        $this->addField($port = new InputPageDigitstringField("playerport", tl::tl("Voice Player Port"), $reportingSession->reportingCfg->voicePlayer->port));
        $port->emptyOk = true;
        $this->addField($button = new ButtonVoicePlayerConfig("buttonvoiceplayer", tl::tl("Ok")));
    }        
    
    public function tabPbx() {
        $this->currentSection = $this->parent->currentTab;
        /* @var $reportingSession ReportingSession */global $reportingSession;
        $options = array(PbxConfig::identifierPbx => tl::tl("PBX Name"),
                    PbxConfig::identifierSys => tl::tl("System Name"));
        $this->addField($identifier = new InputPageDropdownField("ident", tl::tl("Grouping ID"), $options));
        $identifier->default[$reportingSession->reportingCfg->pbx->identifier] = true;
        $this->addField($button = new ButtonPbxConfig("buttonpbx", tl::tl("Ok")));
    }
    
    public function tabFilter() {
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        global $reportingSession;
        // fields shown in the green field under the tab menu
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
        } else {
            if(!$this->parent->showFilter) {
                // fields shown in the green field under the tab menu
                $createName = tl::tl("Add");
                $create = $this->addField(new InputPageText("createfilterlink", $this->getModeLink(ReportingConfigConsts::modeFilter, $createName, "", "javascript:submitFilter(0, '".ReportingConfigConsts::modeFilter."')")));
                $create->fullline = true;
                if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
                    $create->value = disable_link($createName, tl::tl("innovaphone Reporting is running in standby mode, so no filter editing is possible."));
                }
            }
            switch($this->parent->currentMode) {
                case ReportingConfigConsts::modeFilter:
                    $this->modeFilter();
                    break;
                case ReportingConfigConsts::modeDeleteFilter:
                    $this->modeDeleteFilter();
                    break;
            }
            if($this->parent->currentMode == "") {
                $this->addFilterList();
            }
        }
    }
    
    function tabReplication() {
        global $reportingSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        
        $options = array(Replication::modeOff => tl::tl("Off"), Replication::modeMaster => tl::tl("Master"), Replication::modeStandby => tl::tl("Standby"));
        $drop = $this->addField(new InputPageDropdownField("replmode", tl::tl("Mode"), $options));
        $drop->default[$reportingSession->reportingCfg->replication->mode] = true;
        $drop->fullline = true;
        $server = $this->addField(new InputPageIPField("server", tl::tl("Master/Standby Server"), $reportingSession->reportingCfg->replication->server));
        $server->emptyOk = true;
        $server->setHint(tl::tl("Another server with an innovaphone Reporting installation, which shall act as master or standby server."));
        
        if($reportingSession->reportingCfg->replication->server != "") {
            $serverStatus = $this->addField(new InputPageText("serverstatus", ""));
            if(($fp = fsockopen($reportingSession->reportingCfg->replication->server, 5432, $errno, $errstr, 5)) === FALSE) {
                $serverStatus->value = "<div style=\"color:red\">".tl::tl("Server Down")."</div>";
            } else {
                fclose($fp);
                $link = new PostgreSQLLink($reportingSession->reportingCfg->replication->server, "innovaphone-reporting", $reportingSession->reportingCfg->replication->password, 5432, "innovaphone-reporting");
                if($link->isOpen) {
                    $serverStatus->value = tl::tl("Up");
                }
                else {
                    if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby)
                        $serverStatus->value = "<div style=\"color:red\">".tl::tl("Can't connect to Master Server (authentication failed or replication not configured on remote server)")."</div>";
                    else
                        $serverStatus->value = "<div style=\"color:red\">".tl::tl("Can't connect to Standby Server (authentication failed or replication not configured on remote server)").$errorMsg."</div>";
                }
                $link->close();
            }
        } else
            $server->fullline = true;
        $password = $this->addField(new InputPagePasswordField("password", tl::tl("Database password"), $reportingSession->reportingCfg->replication->password));
        $password->minlength = 1;
        $password->emptyOk = true;
        $password->setHint(tl::tl("The PostgreSQL innovaphone-reporting password."));
        $password->fullline = true;
        $interval = $this->addField(new CronInputPageDigitStringField("interval", tl::tl("Synchronisation interval"), $reportingSession->reportingCfg->replication->interval));
        $interval->emptyOk = true;
        $interval->setHint(tl::tl("This is the interval in minutes, in which database records are updated."));
        $interval->fullline = true;
            
        $java = "changeReplicationMode('".Replication::modeOff."', '{$drop->name}', '{$server->name}', '{$password->name}', '{$interval->name}');";
        $drop->attributes["onChange"] = "javascript:$java";
        $drop->setJavaScriptInit("function JavaScriptInit_{$drop->name}() { $java }");
        
        $btn = $this->addField(new ButtonReplicationConfig("savereplicationconfig", tl::tl("Ok")));
        $btn->fullline = true;
        
        $dummy = $this->addField(new InputPageText("dummy", "<br>"));
        $dummy->fullline = true;
        
        $list = $this->addField(new MainInputPageListField("replicationstatuslist"));
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<i>".tl::tl("No replication status yet")."</i>");
        $time = $list->addColumn(new MainInputPageNoBoxDateField($list->name."time", null, 0, null, true), tl::tl("Last Run"));
        $time->sortType = genericInputPageItem::sortIgnore;
        $id = $list->addColumn(new InputPageText($list->name."id", null), tl::tl("Last remote CDR id"));
        $id->sortType = genericInputPageItem::sortIgnore;
        $status = $list->addColumn(new InputPageText($list->name."status", null), tl::tl("Status"));
        $status->sortType = genericInputPageItem::sortIgnore;
        $list->fullline = true;
            
        switch($this->parent->currentMode) {
            case "":
                $this->addSection($this->currentSection."_bottom", 1, false);
                $this->currentSection = $this->currentSection."_bottom";
                $log = $this->addField(new InputPageText("replicationlog", ""));
                parse_log_file(Config::reportingReplicationLog, $logVal, $error, false, true, 100);
                $log->value = $logVal;
                break;
        }
    }
    
    function tabLdap() {
        global $reportingSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
        } else {
            $createName = tl::tl("Add");
            $create = $this->addField(new InputPageText("createldaplink", $this->getModeLink(ReportingConfigConsts::modeLdap, tl::tl("Add"), "", "javascript:submitLdap('0', '".ReportingConfigConsts::modeLdap."')")));
            $create->fullline = true;
            if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
                $create->value = disable_link($createName, tl::tl("innovaphone Reporting is running in standby mode, so no ldap server editing is possible."));
            }
                
            switch($this->parent->currentMode) {
                case ReportingConfigConsts::modeLdap:
                    $this->modeLdap();
                    break;
                case ReportingConfigConsts::modeDeleteLdap:
                    $this->modeDeleteLdap();
                    break;
            }
            if($this->parent->currentMode == "") {
                $this->addLdapList();
            }
        }
    }

    function tabUsers() {
        global $reportingSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        global $reportingSession;
        // fields shown in the green field under the tab menu
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
        } else {
            $createName = tl::tl("Add");
            $create = $this->addField(new InputPageText("createuserlink", $this->getModeLink(ReportingConfigConsts::modeUser, $createName, "", "javascript:submitUser('0', '".ReportingConfigConsts::modeUser."')")));
            $create->fullline = true;
            $link = $this->addField(new InputPageLinkField("userlink", "user/login.php", tl::tl("User Login Page")));
            $link->attributes["target"] = "_blank";
            if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
                $create->value = disable_link($createName, tl::tl("innovaphone Reporting is running in standby mode, so no user editing is possible."));
            }
            
            switch($this->parent->currentMode) {
                case ReportingConfigConsts::modeUser:
                    $this->modeUser();
                    break;
                case ReportingConfigConsts::modeDeleteUser:
                    $this->modeDeleteUser();
                    break;
            }
            if($this->parent->currentMode == "") {
                $this->addUserList();
            }
        }        
    }
    
    function tabImages() {
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();

        if($this->parent->currentMode == ReportingConfigConsts::modeDeleteImage) {
            $this->modeDeleteImage();
        }
        else {
            $imagesUploaded = array();
            if ($handle = opendir(Config::imagesDir)) {
                /* This is the correct way to loop over the directory. */
                while (false !== ($file = readdir($handle))) {
                    if(($file !== ".") && ($file !== ".."))
                        $imagesUploaded[htmlentities_utf8($file)] = htmlentities_utf8($file);
                }        
                closedir($handle);
            }
     
            if(count($imagesUploaded)) {
                $listImages = $this->addField(new InputPageDropdownField("listimages", tl::tl("Images"), $imagesUploaded));
                $listImages->default[$this->imageName] = true;
                $listImages->button = $this->addField(new ButtonAskDeleteImage("buttondeleteimage", tl::tl("Delete")));

                $dummy3 = $this->addField(new InputPageText("dummytext3", "<br>"));
                $dummy3->fullline = true;
            }

            $text->value = tl::tl("Select a Logo or Footer to be printed in PDF reports.");
            $upload = $this->addField(new InputPageFileUploadField("upload", tl::tl("New Image")));
            $upload->button = &$this->addField(new ButtonUploadImage("buttonupload", tl::tl("Upload")));
            $upload->emptyOk = true;
        }
    }

    function tabMail() {
        global $reportingSession;
        $this->currentSection = $this->parent->currentTab;
        $this->addActionText();
        $error = MainMethods::checkRunningScript(Config::configErrorPostgres);
        global $reportingSession;
        // fields shown in the green field under the tab menu
        if($error != "") {
            $running = $this->addField(new InputPageText("procrunning", $error));
            $running->fullline = true;
        } else {
            $createName = tl::tl("Add");
            $create = $this->addField(new InputPageText("createmaillink", $this->getModeLink(ReportingConfigConsts::modeMail, $createName, "", "javascript:submitMail('0', '".ReportingConfigConsts::modeMail."')")));
            $create->fullline = true;
            if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
                $create->value = disable_link($createName, tl::tl("innovaphone Reporting is running in standby mode, so no report mail editing is possible."));
            }
            
            switch($this->parent->currentMode) {
                case ReportingConfigConsts::modeMail:
                    $this->modeMail();
                    break;
                case ReportingConfigConsts::modeDeleteMail:
                    $this->modeDeleteMail();
                    break;
            }
            if($this->parent->currentMode == "") {
                $this->addMailList();
            }
        }
    }
    
    public function modeCDRCleanup($text) {
        /** @var $reportingSession ReportingSession*/global $reportingSession;
        $this->currentSection = $this->parent->currentMode;
        $this->addSection($this->parent->currentMode, 1, true, $text);
        
        $this->addField($txt = new InputPageText("txt", tl::tl("If you activate scheduled cleanup, CDRs older than the configured amount of months + weeks will be automatically deleted.")));
        $this->addSubSection($this->parent->currentMode, "_cfg", 1);
        $this->addField($chk = new InputPageCheckboxField("active", tl::tl("Active")));
        $this->addField($months = new InputPageDigitstringField("months", tl::tl("Months"), strval($reportingSession->reportingCfg->cleanup->months)));
        $this->addField($months = new InputPageDigitstringField("weeks", tl::tl("Weeks"), strval($reportingSession->reportingCfg->cleanup->weeks)));

        $monthsVal = InputPage::formInput($months->name);
        if(!isset($monthsVal)) {
            $chk->default["on"] = $reportingSession->reportingCfg->cleanup->active;
        }
        
        $this->addField($btn = new ButtonConfigureCleanup("buttoncleanup", ReportingConfigConsts::pageConfig, $this->parent));
        $chk->setButtons($btn->name, $btn->name."_cancel");
        $months->setButtons($btn->name, $btn->name."_cancel");
    }
    
    function modeAskDeleteCdrs() {
        global $reportingSession;
        $this->currentSection = PostgresPage::modeAskDeleteCdrs;
        if($this->parent->currentMode == PostgresPage::modeAskDeleteCdrs) {
            $this->addSection(PostgresPage::modeAskDeleteCdrs, 2, true, tl::tl("Define your CDR deletion options"));
        }
        $dateStart = $this->addField(new InputPageCalendarDateField("from", tl::tl("From")));
        $dateEnd = $this->addField(new InputPageCalendarDateField("to", tl::tl("To")));
        $pbxObject = $this->parent->addObjectAndFilterList($this, "%", true, false);
        $pbxObject->setHint(tl::tl("If you enter just '%', CDRs by all pbx objects are deleted."));
        if($this->parent->currentMode == PostgresPage::modeAskDeleteCdrs) {
            $this->addSubSection(PostgresPage::modeAskDeleteCdrs, "_bottom");
        }
        $btn = $this->addField(new ButtonAskDeleteCDRs("buttonaskdeletecdrs", ReportingConfigConsts::pageConfig, $this->parent, PostgresPage::modeDeleteCdrs, ""));
    }
    
    function modeDeleteCdrs() {
        global $reportingSession;
        $this->currentSection = PostgresPage::modeDeleteCdrs;
        $disable = false;
        if($this->parent->currentMode == PostgresPage::modeDeleteCdrs) {
            $this->addSection(PostgresPage::modeDeleteCdrs, 1, true, tl::tl("Delete the CDRs?"));
            $text = $this->addField(new InputPageText("text", ""));
            $from = strtotime(InputPage::formInput("from"));
            $to = strtotime(InputPage::formInput("to"));
            $pbx = InputPage::formInput(ReportingInputPage::valPbx);
            $pbx = $pbx[0];
            $pbxObject = InputPage::formInput(ReportingInputPage::valObject);
            $reportingDb = get_reporting_db();
            $query = "SELECT COUNT(cdrs.id) AS count 
                        FROM cdrs 
                        JOIN cdr_users cu ON cdrs.cdr_user_id=cu.id
                        WHERE ".ButtonDeleteCDRs::QueryConditions($pbxObject, $pbx, $from, $to);
            $result = $reportingDb->run($query);
            if(($row = pg_fetch_object($result)) && $row->count > 0) {
                $text->value = tl::tlx(/* {$arg[1]} is a number, {$arg[2]} a name and {$arg[3]} and {$arg[4]} dates */ 'You are about to delete {$arg[1]} CDRs from pbx object(s) \'{$arg[2]}\' between {$arg[3]} and {$arg[4]}.',
                                        "<b>{$row->count}</b>", "$pbxObject / $pbx", InputPageDateField::translateDate($from), InputPageDateField::translateDate($to));
            }
            else {
                $disable = true;
                    $text->value = tl::tlx(/* {$arg[1]} is a name and {$arg[2]} and {$arg[3]} dates */'No CDRs found for pbx object(s) \'{$arg[1]}\' between {$arg[2]} and {$arg[3]}.',
                                        "$pbxObject / $pbx", InputPageDateField::translateDate($from), InputPageDateField::translateDate($to));
            }
        }
        $btn = $this->addField(new ButtonDeleteCDRs("buttondeletecdrs", ReportingConfigConsts::pageConfig, $this->parent, "", PostgresPage::modeAskDeleteCdrs));
        if($disable) {
            $btn->disabled = true;
        }
        if($this->parent->currentMode == PostgresPage::modeAskDeleteCdrs) {
            $this->hideSection(PostgresPage::modeDeleteCdrs);
        }
        else {
            $this->hideSection(PostgresPage::modeAskDeleteCdrs);
        }
    }
 
    function modeMail() {
        global $reportingSession;
        $this->currentSection = ReportingConfigConsts::modeMail;
        $this->addSection($this->currentSection, 1, true, tl::tl("Report Mail Parameters"));
        
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $hint = $this->addField(new InputPageText("standbyhint", tl::tl("innovaphone Reporting is running in standby mode, so no report mail editing is possible.")."<br><br>"));            
            $hint->fullline = true;
            $hint->attributes["style"] = "color:red";
        }
        
        $mail = null;
        if($this->parent->mailId > 0) {
            $mail = $reportingSession->reportMail;
        }
        else {
            $mail = new ReportMail();
        }

        $mailName = $this->addField(new InputPageStringField("mailname", tl::tl("Name"), $mail->name));
        $mailName->emptyOk = true;
        $addresses = $this->addField(new ReportingMailsField("addresses", tl::tl("Mail Addresses"), $mail->mails));
        $addresses->attributes["maxlength"] = "1024";
        $addresses->hint = tl::tl("Comma separated list of email addresses");
        $addresses->emptyOk = true;
        $this->addField($sender = new InputPageMailField(ReportingInputPage::valSender, tl::tl("Sender E-Mail Address"), $mail->sender));
        $sender->emptyOk = true;
        $this->addField($bulk = new InputPageCheckboxField(ReportingInputPage::valBulk, tl::tl("User Reports")));
        $bulk->hint = tl::tl("Check if each matching user shall receive an own report for the own calls.");
        $pbxObject = $this->parent->addObjectAndFilterList($this, $mail->pbxObject);
        $pbx = $this->parent->getField(ReportingInputPage::valPbx);
        $pbx->default = array();
        $pbx->default[$mail->pbx] = true;
        $groupBy = $this->parent->addFieldGroupBy($this, $mail->groupBy);
        $sortBy = $this->parent->addFieldSortBy($this, $mail->sortBy);
        $sortOrder = $this->parent->addFieldCheckBoxSortOrder($this, $mail->sortOrder);
        $hideDigits = $this->parent->addFieldHideDigit($this, $mail->hideDigits);
        $outputFormats = array(ReportingConfigConsts::outFormatPdf => "PDF", 
		ReportingConfigConsts::outFormatXml => "XML",
	       ReportingConfigConsts::outFormatCsv => "CSV");
        $outFormat = $this->addField(new InputPageDropdownField("outputformat", tl::tl("Output Format"), $outputFormats));
        $outFormat->default[$mail->format] = true;
        $outFormat->fullline = true;
        $compress = $this->addField(new InputPageCheckboxField("compress", tl::tl("Compress file")));
        $anonym = $this->parent->addFieldAnonym($this);
        
        $dispDuration = $this->parent->addFieldDisplayedDuration($this, $mail->displayedDuration);
        $mailNameValue = InputPage::formInput($mailName->name);
        if(!isset($mailNameValue)) {
            $compress->default["on"] = $mail->compress;
            $anonym->default["on"] = $mail->anonym;
            $bulk->default["on"] = $mail->bulk;
        }
        $langs = array();
        foreach (tl::languages() as $shorthand => $path) {
            $langs[$shorthand] = tl::iso639($shorthand);
        }
        $this->addField($lang = new InputPageDropdownField("maillang", tl::tl("Language"), $langs));
        $lang->default[$mail->lang] = true;
        $lang->fullline = true;
        $time = $this->addField(new ReportingHourMinutesField("time", tl::tl("Send time"), sprintf("%02d:%02d", $mail->timeHour, $mail->timeMinute)));
        
        $intervals = array(ReportMail::intervalDaily => tl::tl("Daily"), 
            ReportMail::intervalWeekly => tl::tl("Weekly"),
            ReportMail::intervalMonthly => tl::tl("Monthly"));
        $interval = $this->addField(new InputPageDropdownField("interval", tl::tl("Interval"), $intervals));
        $interval->default[$mail->interval] = true;
        $interval->fullline = true;
        $sectionDayOfWeek = ReportingConfigConsts::modeMail."_dayofweek";
        $sectionDayOfMonth = ReportingConfigConsts::modeMail."_dayofmonth";
        $java = "toggleMailInterval('{$interval->name}[]', '$sectionDayOfWeek', '$sectionDayOfMonth')";
        $interval->attributes["onChange"] = "javascript:$java";
        $interval->setJavaScriptInit("function JavaScriptInit_{$interval->name}() { $java }");
        $this->addSubSection(ReportingConfigConsts::modeMail, "_dayofweek", 1, false, true);
        $daysOfWeek = array();
        $daysOfWeek["1"] = tl::tl("Monday");
        $daysOfWeek["2"] = tl::tl("Tuesday");
        $daysOfWeek["3"] = tl::tl("Wednesday");
        $daysOfWeek["4"] = tl::tl("Thursday");
        $daysOfWeek["5"] = tl::tl("Friday");
        $daysOfWeek["6"] = tl::tl("Saturday");
        $daysOfWeek["0"] = tl::tl("Sunday");
        $dayOfWeek = $this->addField(new InputPageDropdownField("dayofweek", tl::tl("Send day"), $daysOfWeek));
        $dayOfWeek->default[$mail->dayOfWeek] = true;
        $this->addSubSection(ReportingConfigConsts::modeMail, "_dayofmonth", 1, false, true);
        $daysOfMonth = array();
        for($i = 1; $i < 29; $i++) {
            $daysOfMonth["$i"] = sprintf("%02d", $i);
        }
        $dayOfMonth = $this->addField(new InputPageDropdownField("dayofmonth", tl::tl("Send day"), $daysOfMonth));
        $dayOfMonth->default[$mail->dayOfMonth] = true;
        $this->addSubSection(ReportingConfigConsts::modeMail, "_btn", 1);
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $button = $this->addField(new ButtonCancel("cancelmail", tl::tl("Cancel"), ReportingConfigConsts::pageConfig));
            $button->autorender = true;
        }
        else {
            $btn = $this->addField(new ButtonMailConfig("savemailconfig", ReportingConfigConsts::pageConfig, $this->parent));
            if($this->parent->mailId > 0) {
                $del = $this->addField(new ButtonAskDeleteMail("buttondeletemail", tl::tl("Delete")));
            }
            $mailName->attributes["onKeyDown"] = $addresses->attributes["onKeyDown"] = $pbxObject->attributes["onKeyDown"] =
            $groupBy->attributes["onKeyDown"] = $addresses->attributes["onKeyDown"] = $hideDigits->attributes["onKeyDown"] =
            $sortBy->attributes["onKeyDown"] = $sortOrder->attributes["onKeyDown"] = $anonym->attributes["onKeyDown"] =
            $bulk->attributes["onKeyDown"] = $sender->attributes["onKeyDown"] =
            "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
        }
    }
    
    function modeDeleteMail() {
        $this->currentSection = ReportingConfigConsts::modeDeleteMail;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected report mail \'{$arg[1]}\'?', htmlentities_utf8($this->parent->mailName)));
        $del = $this->addField(new ButtonDeleteMail("buttondeletemail", ReportingConfigConsts::pageConfig, $this->parent, "", ReportingConfigConsts::modeMail));
    }
    
    function modeUser() {
        global $reportingSession;
        $this->currentSection = ReportingConfigConsts::modeUser;
        $this->addSection($this->currentSection, 1, true, tl::tl("User Configuration"));
        
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $hint = $this->addField(new InputPageText("standbyhint", tl::tl("innovaphone Reporting is running in standby mode, so no user editing is possible.")."<br><br>"));            
            $hint->fullline = true;
            $hint->attributes["style"] = "color:red";
        }
        
        $user = null;
        if($this->parent->userId > 0) {
            $user = $reportingSession->user;
            $text = $this->addField(new InputPageText("pwhint", tl::tl("Leave the password field empty to maintain the current password.")."<br>"));
        }
        else {
            $user = new ReportingUser();
        }

        $userName = $this->addField(new InputPageStringField("username", tl::tl("Name"), $user->name));
        $userName->emptyOk = true;
        $password = $this->addField(new InputPagePasswordField("password", tl::tl("Password")));
        $password->emptyOk = true;
        
        $text = $this->addField(new InputPageText("basehint", ""));
        $text->value = tl::tl("Select one or more base filters for the user. The reports of the user will be based on one or more of these assigned base filters.");
        $text->attributes["style"] = "margin-top:10px;margin-bottom:10px";       
        
        $this->parent->addObjectAndFilterList($this, "", false);
        
        $this->addSubSection(ReportingConfigConsts::modeUser, "_btn", 1);
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $button = $this->addField(new ButtonCancel("canceluser", tl::tl("Cancel"), ReportingConfigConsts::pageConfig));
            $button->autorender = true;
        }
        else {
            $btn = $this->addField(new ButtonUserConfig("saveuserconfig", ReportingConfigConsts::pageConfig, $this->parent));
            if($this->parent->userId > 0) {
                $del = $this->addField(new ButtonAskDeleteUser("buttondeleteuser", tl::tl("Delete")));
            }
            $userName->attributes["onKeyDown"] = $password->attributes["onKeyDown"] =
            "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
        }
    }
    
    function modeDeleteUser() {
        $this->currentSection = ReportingConfigConsts::modeDeleteUser;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected user \'{$arg[1]}\'?', htmlentities_utf8($this->parent->userName)));
        $reportingDb = get_reporting_db();
        $query = "SELECT COUNT(f.id) AS c 
                    FROM filters f
                    JOIN user_filters uf ON (uf.filter_id=f.id)
                    WHERE uf.user_id={$this->parent->userId} AND f.base_filter IS NOT NULL";
        $result = $reportingDb->run($query);
        $row = pg_fetch_object($result);
        if($row->c > 0) {
            $text = $this->addField(new InputPageText("textfilter", "<div style='color:red'>".tl::tlx('If you delete this user, you will delete {$arg[1]} user created filter(s) too!', $row->c)."</div>"));
        }
        $query = "SELECT COUNT(m.id) AS c 
                    FROM mails m WHERE m.user_id={$this->parent->userId}";
        $result = $reportingDb->run($query);
        $row = pg_fetch_object($result);
        if($row->c > 0) {
            $text = $this->addField(new InputPageText("textmail", "<div style='color:red'>".tl::tlx('If you delete this user, you will delete {$arg[1]} user created mail report(s) too!', $row->c)."</div>"));
        }
        $del = $this->addField(new ButtonDeleteUser("buttondeleteuser", ReportingConfigConsts::pageConfig, $this->parent, "", ReportingConfigConsts::modeUser));
    }
    
    function modeDeleteLdap() {
        $this->currentSection = ReportingConfigConsts::modeDeleteLdap;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected LDAP server \'{$arg[1]}\'?', htmlentities_utf8($this->parent->ldapPbx)));
        $del = $this->addField(new ButtonDeleteLdap("buttondeleteldap", ReportingConfigConsts::pageConfig, $this->parent, "", ReportingConfigConsts::modeLdap));
    }
    
    function modeLdap() {
        global $reportingSession;
        $this->currentSection = ReportingConfigConsts::modeLdap;
        $this->addSection($this->currentSection, 1, true, tl::tl("LDAP Parameters"));
        
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $hint = $this->addField(new InputPageText("standbyhint", tl::tl("innovaphone Reporting is running in standby mode, so no ldap server editing is possible.")."<br><br>"));            
            $hint->fullline = true;
            $hint->attributes["style"] = "color:red";
        }
        
        $ldapServer = null;
        if($this->parent->ldapId > 0) {
            $ldapServer = $reportingSession->ldapServer;
        }
        else {
            $ldapServer = new LdapServer();
        }
        $pbxValue = InputPage::formInput("pbx");
        $state = $this->addField(new InputPageCheckBoxField("state", tl::tl("Active")));
        if(!isset($pbxValue)) {
            $state->default["on"] = $ldapServer->active;
        }
        $pbx = $this->addField(new InputPageStringField("pbx", tl::tl("PBX Name"), $ldapServer->pbx));
        $pbx->emptyOk = true;
        $pbx->setHint(tl::tl("This name identifies the local PBX. It should be the same as configured in objects with type 'PBX' as name."));
        $host = $this->addField(new InputPageIPField("ldaphost", tl::tl("Host"), $ldapServer->host));
        $host->emptyOk = true;
        $port = $this->addField(new InputPageDigitStringField("ldapport", tl::tl("Port"), $ldapServer->port));
        $port->minlength = 1;
        $port->emptyOk = true;
        $tls = $this->addField(new InputPageCheckBoxField("tls", tl::tl("TLS")));
        if(!isset($pbxValue) && strpos($ldapServer->server, "ldaps") !== false) {
            $tls->default["on"] = true;
        }
        $user = $this->addField(new InputPageStringField("ldapuser", tl::tl("User"), $ldapServer->user));
        $user->emptyOk = true;
        $user->attributes["maxlength"] = 128;
        $password = $this->addField(new InputPagePasswordField("ldappassword", tl::tl("Password"), $ldapServer->password));
        $password->minlength = 1;
        $password->emptyOk = true;
        $base = $this->addField(new InputPageStringField("base", tl::tl("Base"), $ldapServer->base));
        $base->emptyOk = true;
        $filter = $this->addField(new InputPageStringField("filter", tl::tl("Object Filter"), $ldapServer->filter));
        $filter->emptyOk = true;
        $filter->setHint(tl::tl("Enter an object filter like '(node=telbook)', e.g. if you use an innovaphone PBX as LDAP server."));
        $nameAttrib = $this->addField(new InputPageStringField("nameattrib", tl::tl("Name Attributes"), $ldapServer->nameAttrib));
        $nameAttrib->emptyOk = true;
        $nameAttrib->setHint(tl::tl("Enter Name Attributes here comma separated."));
        $numberAttrib = $this->addField(new InputPageStringField("numberattrib", tl::tl("Number Attributes"), $ldapServer->numberAttrib));
        $numberAttrib->emptyOk = true;
        $numberAttrib->setHint(tl::tl("Enter Number Attributes here comma separated."));
        $this->addField(new InputPageHorizontalRule());
        $cc = $this->addField(new ReportingDigitStringField("countrycode", tl::tl("Country Code"), $ldapServer->cc));
        $cc->minlength = 1;
        $cc->emptyOk = true;
        $ac = $this->addField(new ReportingDigitStringField("areacode", tl::tl("Area Code"), $ldapServer->ac));
        $ac->minlength = 1;
        $ac->emptyOk = true;
        $np = $this->addField(new ReportingDigitStringField("nationalprefix", tl::tl("National Prefix"), $ldapServer->np));
        $np->minlength = 1;
        $np->emptyOk = true;
        $ip = $this->addField(new ReportingDigitStringField("internationalprefix", tl::tl("International Prefix"), $ldapServer->ip));
        $ip->minlength = 1;
        $ip->emptyOk = true;
        $el = $this->addField(new ReportingDigitStringField("extline", tl::tl("External Line"), $ldapServer->el));
        $el->minlength = 1;
        $el->emptyOk = true;
        $sn = $this->addField(new ReportingDigitStringField("subsnumbers", tl::tl("Subscriber Numbers"), $ldapServer->sn));
        $sn->minlength = 1;
        $sn->emptyOk = true;

        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $button = $this->addField(new ButtonCancel("cancelldap", tl::tl("Cancel"), ReportingConfigConsts::pageConfig));
            $button->autorender = true;
        }
        else {
            $btn = $this->addField(new ButtonLdapConfig("saveldapconfig", ReportingConfigConsts::pageConfig, $this->parent));
            if($this->parent->ldapId > 0) {
                $del = $this->addField(new ButtonAskDeleteLdap("buttondeleteldap", tl::tl("Delete")));
            }
            $pbx->attributes["onKeyDown"] = $host->attributes["onKeyDown"] = $port->attributes["onKeyDown"] =
            $user->attributes["onKeyDown"] = $password->attributes["onKeyDown"] = $base->attributes["onKeyDown"] =
            $filter->attributes["onKeyDown"] =
            $nameAttrib->attributes["onKeyDown"] = $numberAttrib->attributes["onKeyDown"] = $cc->attributes["onKeyDown"] =
            $ac->attributes["onKeyDown"] = $np->attributes["onKeyDown"] = $ip->attributes["onKeyDown"] =
            $el->attributes["onKeyDown"] = $sn->attributes["onKeyDown"] =
            "javascript:submitButton(event, '{$btn->name}', '{$btn->name}_cancel')";
        }
    }
        
    function addFilterList() {
        // fields below the green field, here the filter list
        global $reportingSession;
        $this->addSection(ReportingConfigConsts::tabFilter."_bottom", 1, false);
        $this->currentSection = ReportingConfigConsts::tabFilter."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listfilter"));
        $emptyText = $this->parent->currentMode == ReportingReportConsts::modeCreateReport ? tl::tl("No filters selected") : tl::tl("No filters available");
        $empty = $list->fieldOnEmptyTable = new InputPageText("listfilteremptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $name = $list->addColumn( new InputPageText($list->name."name", null), tl::tl("Name"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $desc = $list->addColumn( new InputPageText($list->name."desc", null), tl::tl("Description"));
        $desc->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $baseFilter = $list->addColumn( new InputPageText($list->name."base", null), tl::tl("Base Filter"));
        $baseFilter->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        if(!$reportingSession->isUser()) {
            $user = $list->addColumn(new InputPageText($list->name."user", null), tl::tl("Users"));
            $user->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        }
        $id = $list->addColumn(new InputPageHiddenIdField($list->name."id"));
    }
    
    function addLdapList() {
        // fields below the green field, here the filter list
        $this->addSection(ReportingConfigConsts::tabLdap."_bottom", 1, false);
        $this->currentSection = ReportingConfigConsts::tabLdap."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listldap"));
        $emptyText = tl::tl("No LDAP servers yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText("listldapemptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $pbx = $list->addColumn( new InputPageText($list->name."pbx", null), tl::tl("PBX"));
        $pbx->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $state = $list->addColumn( new InputPageText($list->name."state", null), tl::tl("Active"));
        $state->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $connected = $list->addColumn( new InputPageText($list->name."connected", null), tl::tl("Status"));
        $connected->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $id = $list->addColumn(new InputPageHiddenIdField($list->name."id"));
    }
    
    function addMailList() {
        // fields below the green field, here the filter list
        global $reportingSession;
        $this->addSection(ReportingConfigConsts::tabMail."_bottom", 1, false);
        $this->currentSection = ReportingConfigConsts::tabMail."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listmail"));
        $emptyText = tl::tl("No report mails yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $name = $list->addColumn( new InputPageText($list->name."name", null), tl::tl("Name"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        if(!$reportingSession->isUser()) {
            $user = $list->addColumn( new InputPageText($list->name."user", null), tl::tl("User"));
            $user->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        }
        $id = $list->addColumn(new InputPageHiddenIdField($list->name."id"));
    }
    
    function addUserList() {
        // fields below the green field, here the filter list
        $this->addSection(ReportingConfigConsts::tabUsers."_bottom", 1, false);
        $this->currentSection = ReportingConfigConsts::tabUsers."_bottom";
        
        $list = $this->addField(new MainInputPageListField("listuser"));
        $emptyText = tl::tl("No users yet");
        $empty = $list->fieldOnEmptyTable = new InputPageText($list->name."emptytable", 
            "<b>$emptyText</b>");
        $empty->style = $this->currentSection;
        $name = $list->addColumn( new InputPageText($list->name."name", null), tl::tl("Name"));
        $name->sortType = genericInputPageItem::sortAlphanumericCaseIndependent;
        $id = $list->addColumn(new InputPageHiddenIdField($list->name."id"));
    }

    public function modeFilter() {
        global $reportingSession;
        
        $this->currentSection = ReportingConfigConsts::modeFilter;
        $this->addSection($this->currentSection, 2, true, tl::tl("Filter Parameters"));
        
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby) {
            $hint = $this->addField(new InputPageText("standbyhint", tl::tl("innovaphone Reporting is running in standby mode, so no filter editing is possible.")."<br><br>"));            
            $hint->fullline = true;
            $hint->attributes["style"] = "color:red";
        }
        
        $filter = null;
        if($this->parent->filterId === 0) {
            $filter = new Filter();
        }
        else {
            $filter = $reportingSession->filter;
        }
        $showBaseFilter = true;
        if($filter->id > 0 && $filter->baseFilter == 0 && $reportingSession->isUser()) {
            $showBaseFilter = false;
        }
        if($showBaseFilter) {
            $baseHint = $this->addField(new InputPageText("basehint", ""));
            $baseHint->value = tl::tl("The conditions of the selected base filter will be implicitly matched (AND conjunction)!");
            $baseHint->fullline = true;
            $userId = 0;
            if($reportingSession->isUser()) {
                $userId = $reportingSession->user->id;
            }
            $baseFilters = $this->parent->getFilterListArray(true, $userId, $userId == 0);
            $baseFilter = $this->addField(new InputPageDropdownField("basefilter", tl::tl("Base Filter"), $baseFilters));
            $baseFilter->default[$filter->baseFilter] = true;
            if($this->parent->showFilter) {
                $baseFilter->fullline = true;
            }
            else {
                $linkName = "showfilter";
                $java = "changeBaseFilter('".$baseFilter->name."', '$linkName', '".ReportingConfigConsts::tabFilter."', '".ReportingConfigConsts::modeFilter."');";
                $baseFilter->attributes["onChange"] = "javascript:$java";
                $link = $this->addField(new InputPageLinkField($linkName, "", tl::tl("Show Filter")));
                $link->setJavaScriptInit("function JavaScriptInit_{$link->name}() { $java }");
            }
        }
        
        // fields to fill in
        $filterName = $this->addField(new InputPageStringField("filtername", tl::tl("Filter Name"), $filter->name));
        $filterName->emptyOk = true;
        $filterName->fullline = true;
        
        $filterDesc = $this->addField(new InputPageStringField("filterdesc", tl::tl("Description")));
        $filterDesc->default = $filter->desc;
        $filterDesc->emptyOk = true;
        $filterDesc->fullline = true;
        //$filterDesc->attributes["cols"] = 30;
        
        if(!$reportingSession->isUser()) {
            $this->addSubSection(ReportingConfigConsts::modeFilter, "_anonym");
            $this->parent->addFieldAnonym($this);
        }
        
        $this->addSubSection(ReportingConfigConsts::modeFilter, "_lists", 2);
        // filter list for pbx objects
        $localList = $this->addConditionList("listfilterlocalconds", tl::tl("Local"));
        $localList->fullline = false;
        $localList->tdattributes["valign"] = "top";
        $localList->attributes["style"] = "margin-right:20px;margin-bottom:10px";
        // filter list for remote endpoints
        $remoteList = $this->addConditionList("listfilterremoteconds", tl::tl("Remote"));
        $remoteList->tdattributes["valign"] = "top";
        // filter list for node/pbx
        $pbxList = $this->addConditionList("listfilterpbxconds", tl::tl("PBX"));
        $pbxList->tdattributes["valign"] = "top";
        
        $this->addSubSection(ReportingConfigConsts::modeFilter, "_checks", 2);
        $cStateHint = $this->addField(new InputPageText("callingstatehint", "<br><b>".tl::tl("Call State")."</b>"));
        $cDirHint = $this->addField(new InputPageText("callingdirectionhint", "<br><b>".tl::tl("Call directions")."</b>"));
        //$cTypeHint = $this->addField(new InputPageText("callingtypehint", "<br><b>".tl::tl("Call types")."</b>"));
        
        $cStateAlert = $this->addField(new InputPageCheckboxField("callstatealert", tl::tl("No Response")));
        $cStateAlert->attributes["style"] = "margin-right:20px";
        $cDirIn = $this->addField(new InputPageCheckboxField("calldirin", tl::tl("Incoming")));
        $cDirIn->attributes["style"] = "margin-right:20px";
        //$cTypeInt = $this->addField(new InputPageCheckboxField("calltypeint", tl::tl("Internal")));
        
        $cStateConn = $this->addField(new InputPageCheckboxField("callstateconn", tl::tl("Connected")));
        $cDirOut = $this->addField(new InputPageCheckboxField("calldirout", tl::tl("Outgoing")));
        //$cTypeExt = $this->addField(new InputPageCheckboxField("calltypeext", tl::tl("External")));
        
        $cStateBusy = $this->addField(new InputPageCheckboxField("callstatebusy", tl::tl("Busy")));
        $cDirCt = $this->addField(new InputPageCheckboxField("calldirct", tl::tl("Transfer")));
        $cDirCt->fullline = true;
        
        $cStateNoChannel = $this->addField(new InputPageCheckboxField("callstatenochannel", tl::tl("No Channel")));
        $cDirCf = $this->addField(new InputPageCheckboxField("calldircf", tl::tl("Call Forward")));
        $cDirCf->fullline = true;
        
        $dummy3 = $this->addField(new InputPageText("dummytext3", "<br>"));
        $dummy3->fullline = true;
    
        $this->addSubSection(ReportingConfigConsts::modeFilter, "_compares", 2);
        $callingDurationCompares = array(ReportingConfigConsts::callingDurationGreater => tl::tl("Greater/Equal"), ReportingConfigConsts::callingDurationSmaller => tl::tl("Smaller/Equal"));
        $callingDurationCompare = $this->addField(new InputPageDropdownField("callingdurationcompare", tl::tl("Call Duration (User) (in seconds)"), $callingDurationCompares));
        $callingDurationCompare->default[$filter->callingDurationCompare] = true;
        $callingDuration = $this->addField(new InputPageDigitstringField("callingduration", " ", $filter->callingDuration));
        $callingDuration->emptyOk = true;
        $callingDuration->setHint(tl::tl("Enter the calling duration in seconds."));
                
        $alertDurationCompare = $this->addField(new InputPageDropdownField("alertdurationcompare", tl::tl("Alert Duration (in seconds)"), $callingDurationCompares));
        $alertDurationCompare->default[$filter->alertDurationCompare] = true;
        $alertDuration = $this->addField(new InputPageDigitstringField("alertduration", " ", $filter->alertDuration));
        $alertDuration->emptyOk = true;
        $alertDuration->setHint(tl::tl("Enter the alert duration in seconds."));

        $dummy4 = $this->addField(new InputPageText("dummytext4", "<br>"));
        $dummy4->fullline = true;
    
        // times from to list
        $timeText = $this->addField(new InputPageText("timetext", "<b>".tl::tl("Report times:")."</b>"));
        $timeText->fullline = true;
        $listTimes = $this->addField(new MainInputPageListField("listtimes"));
        $listTimes->sortDirection = InputPageListField::sortNone;
        $listTimes->oddAttributes["class"] = "filter-listfield-tr-separator";
        $listTimes->evenAttributes["class"] = "filter-listfield-tr-separator";
        $listTimes->headAttributes["class"] = "filter-headlist-separator";
        $listTimes->divattributes["class"] = "filter-cond-listfield";
        $days = array(FilterTime::dayUnspecified => tl::tl("Monday")."-".tl::tl("Sunday"),
            FilterTime::dayMonday => tl::tl("Monday"),
            FilterTime::dayTuesday => tl::tl("Tuesday"),
            FilterTime::dayWednesday => tl::tl("Wednesday"),
            FilterTime::dayThursday => tl::tl("Thursday"),
            FilterTime::dayFriday => tl::tl("Friday"),
            FilterTime::daySaturday => tl::tl("Saturday"),
            FilterTime::daySunday => tl::tl("Sunday"),
            FilterTime::dayMonFri => tl::tl("Monday")."-".tl::tl("Friday"),
            FilterTime::daySaSo => tl::tl("Saturday")."-".tl::tl("Sunday"));
        $listTimes->addColumn($listTimesDay = new InputPageDropdownField($listTimes->name."day", null, $days), tl::tl("Weekday(s)"));
        $listTimesDay->sortType = genericInputPageItem::sortIgnore;
        $listTimesDay->default[FilterTime::dayUnspecified] = true;
        $listTimesFrom = $listTimes->addColumn(new ReportingHourMinutesField($listTimes->name."from", null), tl::tl("From"));
        $listTimesFrom->sortType = genericInputPageItem::sortIgnore;
        $listTimesFrom->emptyOk = true;
        $listTimesTo = $listTimes->addColumn(new ReportingHourMinutesField($listTimes->name."to", null), tl::tl("To"));
        $listTimesTo->sortType = genericInputPageItem::sortIgnore;
        $listTimesTo->emptyOk = true;
        $listTimes->removableRows = true;
        
        $imagesLogos = array();
        $imagesFooter = array();
        $imagesLogos[$filter->logo] = htmlentities_utf8($filter->logo);
        $imagesFooter[$filter->footer] = htmlentities_utf8($filter->footer);
        if ($handle = opendir(Config::imagesDir)) {
            /* This is the correct way to loop over the directory. */
            while (false !== ($file = readdir($handle))) {
                if(($file !== ".") && ($file !== "..") && ($file != $filter->logo))
                    $imagesLogos[htmlentities_utf8($file)] = htmlentities_utf8($file);
                if(($file !== ".") && ($file !== "..") && ($file != $filter->footer))
                    $imagesFooter[htmlentities_utf8($file)] = htmlentities_utf8($file);
            }
            closedir($handle);
        }
        if($filter->logo !== "") {
            $imagesLogos[""] = "";
        }
        if($filter->footer !== "") {
            $imagesFooter[""] = "";
        }
        $listLogos = $this->addField(new InputPageDropdownField("listlogo", tl::tl("Logo"), $imagesLogos));
        $listLogos->default[$filter->logo] = true;
        $listFooter = $this->addField(new InputPageDropdownField("listfooter", tl::tl("Footer"), $imagesFooter));
        $listFooter->default[$filter->footer] = true;
    
        if($this->parent->showFilter) return;
        $this->addSubSection(ReportingConfigConsts::modeFilter, "_buttons", 1);
        if($reportingSession->reportingCfg->replication->mode == Replication::modeStandby ||
            ($reportingSession->isUser() && $filter->id > 0 && $filter->baseFilter == 0)) {
            $button = $this->addField(new ButtonCancel("cancelfilter", tl::tl("Cancel"), ReportingConfigConsts::pageConfig));
            $button->autorender = true;
        }
        else {
            $button = $this->addField(new ButtonFilterConfig("buttoncreatefilter", ReportingConfigConsts::pageConfig, $this->parent));
            $button->fullline = true;
            $button->setTdAttribute("style", "padding-top:3px; padding-bottom:10px");
            if($this->parent->filterId > 0) {
                $del = $this->addField(new ButtonAskDeleteFilter("buttondeletefilter", tl::tl("Delete")));
            }
        }
    }
    
    public function addConditionList($name, $head) {
        $list = $this->addField(new MainInputPageListField($name));
        $list->sortDirection = InputPageListField::sortNone;
        $list->oddAttributes["class"] = "filter-listfield-tr-separator";
        $list->evenAttributes["class"] = "filter-listfield-tr-separator";
        $list->headAttributes["class"] = "filter-headlist-separator";
        $list->divattributes["class"] = "filter-cond-listfield";
        
        $listCond = $list->addColumn(new InputPageDropdownField($list->name."select", null, array()), $head);
        $listCond->sortType = genericInputPageItem::sortIgnore;
        $listValue = $list->addColumn(new InputPageListStringField($list->name."value", null), null /*tl::tl("Value")*/);
        $listValue->sortType = genericInputPageItem::sortIgnore;
        $listValue->emptyOk = true;
        $list->removableRows = true;
        return $list;
    }
    
    public function modeDeleteImage() {
        $this->currentSection = ReportingConfigConsts::modeDeleteImage;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected image \'{$arg[1]}\'?', htmlentities_utf8($this->parent->imageName)));
        $del = $this->addField(new ButtonDeleteImage("buttondeleteimage", ReportingConfigConsts::pageConfig, $this->parent, "", ReportingConfigConsts::modeImage));
    }

    public function modeDeleteFilter() {
        $this->currentSection = ReportingConfigConsts::modeDeleteFilter;
        $this->addSection($this->currentSection, 1, true, tl::tlx('Delete the selected filter \'{$arg[1]}\'?', htmlentities_utf8($this->parent->filterName)));
        $reportingDb = get_reporting_db();
        $query = "SELECT COUNT(id) AS c FROM filters WHERE base_filter={$this->parent->filterId}";
        $result = $reportingDb->run($query);
        $row = pg_fetch_object($result);
        if($row->c > 0) {
            $text = $this->addField(new InputPageText("text", "<div style='color:red'>".tl::tlx('If you delete this base filter, you will delete {$arg[1]} depending filter(s) too!', $row->c)."</div>"));
        }
        $del = $this->addField(new ButtonDeleteFilter("buttondeletefilter", ReportingConfigConsts::pageConfig, $this->parent, "", ReportingConfigConsts::modeFilter));
    }
    
    public function fillLists() {
        switch($this->parent->currentTab) {
            case ReportingConfigConsts::tabFilter:
                switch($this->parent->currentMode) {
                    case "": 
                        $this->fillFilterList();
                        break;
                    case ReportingConfigConsts::modeFilter:
                        $this->fillListModeFilter();
                        break;
                }
                break;
            case ReportingConfigConsts::tabPgsql:
                $this->fillIpList("innovaphone-reporting");
                break;
            case ReportingConfigConsts::tabLdap:
                if($this->parent->currentMode == "") {
                    $this->fillLdapList();
                }
                break;
            case ReportingConfigConsts::tabMail:
                switch($this->parent->currentMode) {
                    case "": 
                        $this->fillMailList(); 
                        break;
                    case ReportingConfigConsts::modeMail:
                        $this->fillMailFilterList();
                        break;
                }
                break;
            case ReportingConfigConsts::tabUsers:
                switch($this->parent->currentMode) {
                    case "": 
                        $this->fillUserList();
                        break;
                    case ReportingConfigConsts::modeUser:
                        $this->fillUserFilterList();
                        break;
                }
                break;
            case ReportingConfigConsts::tabReplication:
                $this->fillReplicationList();
                break;
        }
    }
    
    function fillReplicationList() {
        global $reportingSession;
        if($reportingSession->reportingCfg->replication->mode != Replication::modeOff) {
            $reportingDb = get_reporting_db();
            $query = "SELECT * FROM replication";
            $result = $reportingDb->run($query);
            $row = pg_fetch_object($result);
            
            $list = $this->parent->getField("replicationstatuslist");
            $list->value = array();    
            $status = "";
            if(file_exists(Config::reportingReplicationStatus)) {
                $status = file_get_contents(Config::reportingReplicationStatus);
            }
            if($row && $status != "") {
                $list->value[0][$list->name."time"] = strtotime($row->last_replication_time);
                $list->value[0][$list->name."id"] = $row->last_cdr_id;
                $list->value[0][$list->name."status"] = $status === "0" ? tl::tl("Ok") : (tl::tl("Failed")." ($status)");
            }
            else if($status != "") {
                $list->value[0][$list->name."time"] = filemtime(Config::reportingReplicationStatus);
                $list->value[0][$list->name."id"] = "";
                $list->value[0][$list->name."status"] = $status === "0" ? tl::tl("Ok") : (tl::tl("Failed")." ($status)");
            }
        }
    }
    
    function fillLdapList() {
        $list = $this->parent->getField("listldap");
        if(!is_object($list)) return;
        $rows = 0;
        
        $reportingDb = get_reporting_db();
        $query = "SELECT id, server, host, pbx, port, active, username, password FROM ldap_servers";   
        $result = $reportingDb->run($query);
        while($row = pg_fetch_object($result)) {
            $list->value[$rows][$list->name."pbx"] = "<a id=\"".htmlentities_utf8($row->pbx)."\" class=\"myinno-listlink\" href=\"javascript:submitLdap('{$row->id}', '".ReportingConfigConsts::modeLdap."')\">".htmlentities_utf8($row->pbx)."</a>";
            $list->value[$rows][$list->name."state"] = ($row->active == 't' ? tl::tl("Yes") : tl::tl("No"));
            $list->value[$rows][$list->name."id"] = $row->id;
            /*LDAP*/
            if(strstr($row->server, "ldaps")) {
                $conn_string = "ldaps://".$row->host.":".$row->port;
            }
            else {
                $conn_string = "ldap://".$row->host.":".$row->port;
            }
            if(($ldapconn = ldap_connect ($conn_string)) !== FALSE) {
                if(ldap_set_option($ldapconn, LDAP_OPT_PROTOCOL_VERSION, 3)) {
                    if(ldap_set_option($ldapconn, LDAP_OPT_NETWORK_TIMEOUT, 3)) {
                        $ldapbind = ldap_bind($ldapconn, $row->username,  rc4_decrypt(strrev($row->host), $row->password));
                        $list->value[$rows][$list->name."connected"] = $ldapbind ? tl::tl("Up") : tl::tl("Down");
                        if($ldapbind) ldap_unbind($ldapconn);
                    } else {
                        $list->value[$rows][$list->name."connected"] = tl::tl("Down");
                    }
                } else {
                    $list->value[$rows][$list->name."connected"] = tl::tl("Down");
                }
                ldap_close($ldapconn);
            } else {
                $list->value[$rows][$list->name."connected"] = tl::tl("Down");
            }
            $rows++;
        }
    }
    
    function fillMailFilterList() {
        global $reportingSession;
        $list = $this->parent->getField(ReportingInputPage::valList);
        $rows = 0;
        $userId = 0;
        if($reportingSession->isUser()) {
            $userId = $reportingSession->user->id;
        }
        $selects = $this->parent->getFilterListArray(false, $userId, $userId == 0);
            
        $mail = null;
        if($this->parent->mailId > 0) {
            $mail = $reportingSession->reportMail;
        }
        else {
            $mail = new ReportMail();
        }
        if(!$list->sent && count($mail->filters)) {
            foreach($mail->filters as $filterId => $filterName) {
                $list->value[$rows][$list->name."select"][$filterId] = true;
                $list->value[$rows][$list->name."select_options"] = $selects;
                $rows++;
            }
        }
        else if(!count($list->value)) {
            $list->value = array();
            $list->value[0][$list->name."select"][0] = true;
            $list->value[0][$list->name."select_options"] = $selects;
        }
        else {
            unset($vals);
            foreach($list->value as $key => &$vals) {
                $vals[$list->name."select_options"] = $selects;
                $rows++;
            }
        }
    }
    
    function fillUserFilterList() {
        global $reportingSession;
        $list = $this->parent->getField(ReportingInputPage::valList);
        $rows = 0;
        $selects = $this->parent->getFilterListArray(true);
            
        $user = null;
        if($this->parent->userId > 0) {
            $user = $reportingSession->user;
        }
        else {
            $user = new ReportingUser();
        }
        if(!$list->sent && count($user->filters)) {
            foreach($user->filters as $filterId => $filterName) {
                $list->value[$rows][$list->name."select"][$filterId] = true;
                $list->value[$rows][$list->name."select_options"] = $selects;
                $rows++;
            }
        }
        else if(!count($list->value)) {
            $list->value = array();
            $list->value[0][$list->name."select"][0] = true;
            $list->value[0][$list->name."select_options"] = $selects;
        }
        else {
            unset($vals);
            foreach($list->value as $key => &$vals) {
                $vals[$list->name."select_options"] = $selects;
                $rows++;
            }
        }
    }
    
    function fillMailList() {
        global $reportingSession;
        $list = $this->parent->getField("listmail");
        if(!is_object($list)) return;
        $list->value = array();
        $rows = 0;
        
        $reportingDb = get_reporting_db();
        $query = "SELECT m.id, m.name, u.name AS user FROM mails m
                    LEFT OUTER JOIN users u ON (u.id=m.user_id)";   
        if($reportingSession->isUser()) {
            $query .= " WHERE m.user_id={$reportingSession->user->id}";
        }
        $result = $reportingDb->run($query);
        while($row = pg_fetch_object($result)) {
            $list->value[$rows][$list->name."name"] = "<a id=\"".htmlentities_utf8($row->name)."\" class=\"myinno-listlink\" href=\"javascript:submitMail({$row->id}, '".ReportingConfigConsts::modeMail."')\">".htmlentities_utf8($row->name)."</a>";
            $list->value[$rows][$list->name."user"] = (isset($row->user) ? $row->user : "");
            $list->value[$rows][$list->name."id"] = $row->id;
            $rows++;
        }
    }
    
    function fillUserList() {
        $list = $this->parent->getField("listuser");
        if(!is_object($list)) return;
        $rows = 0;
        
        $reportingDb = get_reporting_db();
        $query = "SELECT id, name FROM users";   
        $result = $reportingDb->run($query);
        while($row = pg_fetch_object($result)) {
            $list->value[$rows][$list->name."name"] = "<a id=\"".htmlentities_utf8($row->name)."\" class=\"myinno-listlink\" href=\"javascript:submitUser({$row->id}, '".ReportingConfigConsts::modeUser."')\">".htmlentities_utf8($row->name)."</a>";
            $list->value[$rows][$list->name."id"] = $row->id;
            $rows++;
        }
    }
    
    function fillFilterList() {
        $list = $this->parent->getField("listfilter");
        if(!is_object($list)) return;
        $list->value = array();
        $rows = 0;
        global $reportingSession;
        
        $reportingDb = get_reporting_db();
        $query = "SELECT f.id, f.name, f.description, f2.name AS base, 
                                    array_to_string(array(SELECT u.name FROM users u
					JOIN user_filters uf ON u.id=uf.user_id
                                        WHERE uf.filter_id=f.id
                                        ORDER BY lower(u.name)),',') AS users
                    FROM filters f
                    LEFT OUTER JOIN filters f2 ON (f2.id=f.base_filter)";
        if($reportingSession->isUser()) {
            $query .= " WHERE f.id IN (SELECT uf.filter_id 
                                        FROM user_filters uf WHERE uf.user_id={$reportingSession->user->id})";
        }
        $query .= " ORDER BY lower(f.name)";
        
        $result = $reportingDb->run($query);
        while($row = pg_fetch_object($result)) {
            $list->value[$rows][$list->name."name"] = "<a id=\"".htmlentities_utf8($row->name)."\" class=\"myinno-listlink\" href=\"javascript:submitFilter({$row->id}, '".ReportingConfigConsts::modeFilter."')\">".htmlentities_utf8($row->name)."</a>";
            $list->value[$rows][$list->name."desc"] = htmlentities_utf8($row->description);
            $list->value[$rows][$list->name."id"] = $row->id;
            $list->value[$rows][$list->name."base"] = (isset($row->base) ? $row->base : "");
            $list->value[$rows][$list->name."user"] = (isset($row->users) ? $row->users : "");
            $rows++;
        }
    }
    
    public function fillListModeFilter() {
        global $reportingSession;
        $filter = null;
        if($this->parent->filterId === 0) {
            $filter = new Filter();
        }
        else {
            $filter = $reportingSession->filter;
        }
        $colNames = ReportingConfigConsts::getCondColNames();
        
        $listLocal = $this->parent->getField("listfilterlocalconds");
        if(!is_object($listLocal)) return;
        $selects = array(ReportingConfigConsts::colCCnUser => $colNames[ReportingConfigConsts::colCCnUser],
                ReportingConfigConsts::colCE164 => $colNames[ReportingConfigConsts::colCE164],
                ReportingConfigConsts::colGroups => $colNames[ReportingConfigConsts::colGroups]);
        $this->fillListModeFilterValues($filter, $listLocal, $selects, FilterCondition::typeLocal);
        
        $listRemote = $this->parent->getField("listfilterremoteconds");
        $selects = array(ReportingConfigConsts::colCdE164 => $colNames[ReportingConfigConsts::colCdE164],
                         ReportingConfigConsts::colCdDn => $colNames[ReportingConfigConsts::colCdDn]);
        $this->fillListModeFilterValues($filter, $listRemote, $selects, FilterCondition::typeRemote);
        
        $listPbx = $this->parent->getField("listfilterpbxconds");
        $selects = array(ReportingConfigConsts::colSys => $colNames[ReportingConfigConsts::colSys],
                ReportingConfigConsts::colPbxUser => $colNames[ReportingConfigConsts::colPbxUser],
                ReportingConfigConsts::colNode => $colNames[ReportingConfigConsts::colNode]);
        $this->fillListModeFilterValues($filter, $listPbx, $selects, FilterCondition::typePbx);
        
        $listTimes = $this->parent->getField("listtimes");
        $rows = 0;
        if(!$listTimes->sent && count($filter->times)) {
            foreach($filter->times as $time) {
                $listTimes->value[$rows][$listTimes->name."from"] = $time->getFrom();
                $listTimes->value[$rows][$listTimes->name."to"] = $time->getTo();
                $listTimes->value[$rows][$listTimes->name."day"][$time->day] = true;
                $rows++;
            }
        }
        else if(!count($listTimes->value)) {
            $listTimes->value[$rows][$listTimes->name."from"] = "";
            $listTimes->value[$rows][$listTimes->name."to"] = "";
            $listTimes->value[$rows][$listTimes->name."day"][FilterTime::dayUnspecified] = true;
        }
        else {
        }
    }
    
    public function fillListModeFilterValues(Filter &$filter, MainInputPageListField &$list, $selects, $type) {
        $rows = 0;
        $count = 0;
        
        foreach($filter->conds as $key => $cond) {
            if($cond->type == $type) {
                $count++;
            }
        }
        if(!$list->sent && $count) {
            foreach($filter->conds as $key => $cond) {
                if($cond->type == $type) {
                    $list->value[$rows][$list->name."select"][$cond->name] = true;
                    $list->value[$rows][$list->name."select_options"] = $selects;
                    $list->value[$rows][$list->name."value"] = $cond->value;
                    $rows++;
                }
            }
        }
        else if(!count($list->value)) {
            $first = "";
            foreach($selects as $key => $select) {
                $first = $key;
                break;
            }
            $list->value[$rows][$list->name."select"][$first] = true;
            $list->value[$rows][$list->name."select_options"] = $selects;
            $list->value[$rows][$list->name."value"] = "";
        }
        else {
            unset($vals);
            foreach($list->value as $key => &$vals) {
                $vals[$list->name."select_options"] = $selects;
                $rows++;
            }
        }
    }
}

final class ReportingConfig extends ReportingInputPage {
    var $filterId = 0;
    var $filterName = "";
    var $ldapId = 0;
    var $ldapPbx = "";
    var $mailId = 0;
    var $mailName = "";
    var $userId = 0;
    var $userName = "";
    var $imageName = "";
    var $showFilter = false;
    
    // constructor
    public function ReportingConfig() {
        parent::ReportingInputPage(tl::tl("Reporting Config"), "reportingconfig");
        global $reportingSession;
        $this->setFieldSectionAttributes("inline", new _InputPageFieldSectionAttributes(array(), 1));
        
        if(InputPage::formInput("showfilter") != null) {
            $this->showFilter = true;
        }
        
        if($reportingSession->isUser()) {
            switch($this->currentTab) {
                case "":
                case ReportingConfigConsts::tabFilter:
                case ReportingConfigConsts::tabMail:
                    break;
                default:
                    home_die(tl::tl("You are not allowed to visit this site!"), "report.php");
                    exit;
            }
        }
        
        // dummy fields to save certain parameters over all pages
        $this->addField(new InputPageHiddenIdField("filter_id", $this->filterId));
        $this->filterId = InputPage::formInput("filter_id");
        if(!isset($this->filterId)) {
            $this->filterId = 0;
        }
        else {
            $this->filterId = intval($this->filterId);
        }

        $this->addField(new InputPageHiddenIdField("image_name", $this->imageName));
        $this->imageName = InputPage::formInput("image_name");
        if(!isset($this->imageName)) {
            $this->imageName = "";
        }   

        $this->addField(new InputPageHiddenIdField("mail_id", $this->mailId));
        $this->mailId = InputPage::formInput("mail_id");
        if(!isset($this->mailId)) {
            $this->mailId = 0;
        }
        else {
            $this->mailId = intval($this->mailId);
        }
        
        $this->addField(new InputPageHiddenIdField("user_id", $this->userId));
        $this->userId = InputPage::formInput("user_id");
        if(!isset($this->userId)) {
            $this->userId = 0;
        }
        else {
            $this->userId = intval($this->userId);
        }

        $this->addField(new InputPageHiddenIdField("ldap_id", $this->ldapId));
        $this->ldapId = InputPage::formInput("ldap_id");
        if(!isset($this->ldapId)) {
            $this->ldapId = 0;
        }
        else {
            $this->ldapId = intval($this->ldapId);
        }
        
        switch($this->currentMode) {
            case "":
                $reportingSession->updateFilter(new Filter());
                $reportingSession->updateReportMail(new ReportMail());
                $reportingSession->updateLdapServer(new LdapServer());
                break;
            case ReportingConfigConsts::modeFilter:
            case ReportingConfigConsts::modeDeleteFilter:
                if($reportingSession->filter->id != $this->filterId) {
                    if($this->filterId > 0) {
                        $filter = ReportingMethods::getFilter($this->filterId, "", $error);
                        if($filter != null) {
                            $reportingSession->updateFilter($filter);
                        }
                        else {
                            $reportingSession->updateFilter(new Filter());
                            $this->filterId = 0;
                        }
                    }
                    else {
                        $reportingSession->updateFilter(new Filter());
                    }
                }
                break;
            case ReportingConfigConsts::modeMail:
            case ReportingConfigConsts::modeDeleteMail:
                if($reportingSession->reportMail->id != $this->mailId) {
                    if($this->mailId > 0) {
                        $mail = ReportingMethods::getReportMail($this->mailId, "", $error);
                        if($mail != null) {
                            $reportingSession->updateReportMail($mail);
                        }
                        else {
                            $reportingSession->updateReportMail(new ReportMail());
                            $this->mailId = 0;
                        }
                    }
                    else {
                        $reportingSession->updateReportMail(new ReportMail());
                    }
                }
                break;
            case ReportingConfigConsts::modeUser:
            case ReportingConfigConsts::modeDeleteUser:
                if($reportingSession->user->id != $this->userId) {
                    if($this->userId > 0) {
                        $user = ReportingMethods::getUser($this->userId);
                        if($user != null) {
                            $reportingSession->updateUser($user);
                        }
                        else {
                            $reportingSession->updateUser(new ReportingUser());
                            $this->userId = 0;
                        }
                    }
                    else {
                        $reportingSession->updateUser(new ReportingUser());
                    }
                }
                break;
            case ReportingConfigConsts::modeLdap:
            case ReportingConfigConsts::modeDeleteLdap:
                if($reportingSession->ldapServer->id != $this->ldapId) {
                    if($this->ldapId > 0) {
                        $ldapServer = ReportingMethods::getLdapServer($this->ldapId, $error);
                        if($ldapServer != null) {
                            $reportingSession->updateLdapServer($ldapServer);
                        }
                        else {
                            $reportingSession->updateLdapServer(new LdapServer());
                            $this->ldapId = 0;
                        }
                    }
                    else {
                        $reportingSession->updateLdapServer(new LdapServer());
                    }
                }
                break;
        }
        $this->mailName = $reportingSession->reportMail->name;
        $this->filterName = $reportingSession->filter->name;
        $this->ldapPbx = $reportingSession->ldapServer->pbx;
        $this->userName = $reportingSession->user->name;
        
        // add proper fields to pages
        $this->pages = array(ReportingConfigConsts::pageConfig => new PageConfig($this, false, tl::tl("Reporting Filters")));
        
        // update page
        $this->setDefaultTabProperties();
        $this->pages[$this->pageNo]->createFields();
    }
    
    public function renderPrefix() {
        if($this->showFilter) return "";
        return parent::renderPrefix();
    }
    
    public function renderSuffix() {
        if($this->showFilter) return "";
        return parent::renderSuffix();
    }
    
    public function getArgs() {
        $id = "";
        $ldap = "";
        $mail = "";
        if($this->filterId > 0) {
            $id = "filter_id={$this->filterId}&";
        }
        if($this->ldapId > 0) {
            $ldap = "ldap_id={$this->ldapId}&";
        }
        if($this->mailId > 0) {
            $mail = "mail_id={$this->mailId}&";
        }
        if($this->userId > 0) {
            $user = "user_id={$this->userId}&";
        }
        if($this->imageName !== "") {
            $image = "image_name=".urlencode($this->imageName)."&";
        }
        return "{$image}{$mail}{$ldap}{$id}{$user}tab={$this->currentTab}";
    }
};

// create page
$page = new ReportingConfig;

if($page->asyncMethodField == ReportingInputPage::valObject) {
    if($page->getField(ReportingInputPage::valObject) === null) {
        $page->addField(new ReportingPbxObjectField(ReportingInputPage::valObject, "", ""));
    }
}
else if($page->pageNo == ReportingConfigConsts::pageConfig) {
    $page->updateFields();
    if($page->currentMode == ReportingConfigConsts::modeFilter) {
        $filterName = $page->getField("filtername");
        if(!$filterName->sent) {    // default checkboxes to be set
            global $reportingSession;
            $filter = null;
            if($page->filterId === 0) {
                $filter = new Filter();
            }
            else {
                $filter = $reportingSession->filter;
            }
            
            $chk = $page->getField("callstatealert");
            if($filter->callingState == ReportingConfigConsts::callingStateAll || strpos($filter->callingState, ReportingConfigConsts::callingStateAlert) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("callstateconn");
            if($filter->callingState == ReportingConfigConsts::callingStateAll || strpos($filter->callingState, ReportingConfigConsts::callingStateConn) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("callstatebusy");
            if($filter->callingState == ReportingConfigConsts::callingStateAll || strpos($filter->callingState, ReportingConfigConsts::callingStateBusy) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("callstatenochannel");
            if($filter->callingState == ReportingConfigConsts::callingStateAll || strpos($filter->callingState, ReportingConfigConsts::callingStateNoChannel) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calldirin");
            if($filter->callDirection == ReportingConfigConsts::callDirectionAll || strpos($filter->callDirection, ReportingConfigConsts::callDirectionIn) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calldirout");
            if($filter->callDirection == ReportingConfigConsts::callDirectionAll || strpos($filter->callDirection, ReportingConfigConsts::callDirectionOut) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calldirct");
            if($filter->callDirection == ReportingConfigConsts::callDirectionAll || strpos($filter->callDirection, ReportingConfigConsts::callDirectionCt) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calldircf");
            if($filter->callDirection == ReportingConfigConsts::callDirectionAll || strpos($filter->callDirection, ReportingConfigConsts::callDirectionCf) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calltypeint");
            if(is_object($chk) && $filter->callType == ReportingConfigConsts::callTypeAll || strpos($filter->callType, ReportingConfigConsts::callTypeInt) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField("calltypeext");  
            if(is_object($chk) && $filter->callType == ReportingConfigConsts::callTypeAll || strpos($filter->callType, ReportingConfigConsts::callTypeExt) !== false) {
                $chk->value["on"] = true;
            }
            $chk = $page->getField(ReportingInputPage::valAnonym);
            $chk->value["on"] = $filter->anonym;
        }
    }
    $page->pages[$page->pageNo]->fillLists();
}

$page->work();

close_reporting_db();
?>
